@@ -4,12 +4,12 @@ export PATH
 ##########################################################
 debug=1
 log=1
-configfile=/usr/local/etc/netams.cfg
+configfile=/usr/local/etc/netams.conf
 appfile=/usr/local/libexec/netams
 logfile=netams.log
 path=/var/log
 ##########################################################
-# $Id: netams-startup-failover.sh,v 1.2 2004/11/21 22:23:17 anton Exp $
+# $Id: netams-startup-failover.sh,v 1.3 2008-01-03 12:44:53 anton Exp $
 
 start_loop() {
 
