@@ -1,8 +1,8 @@
 #!/bin/sh
-# $Id: netams-startup.sh,v 1.6 2004/11/27 01:05:19 anton Exp $
+# $Id: netams-startup.sh,v 1.7 2008-01-03 12:44:53 anton Exp $
 # this is FreeBSD version
 # for Linux, replace /usr/local/libexec/netams to /usr/local/sbin/netams
-# and /usr/local/etc/netams.cfg to /etc/netams.cfg
+# and /usr/local/etc/netams.conf to /etc/netams.conf
 daemondir=DAEMONDIR
 path_to_etc=PATH_TO_ETC
 
@@ -12,8 +12,8 @@ start)
         sleep 3;
         /bin/mkdir -p /var/run/netams
         if [ -x $daemondir/netams ]; then
-                if [ -f $path_to_etc/netams.cfg ]; then
-                        $daemondir/netams -lf $path_to_etc/netams.cfg >/dev/null && echo -n ' NeTAMS'
+                if [ -f $path_to_etc/netams.conf ]; then
+                        $daemondir/netams -lf $path_to_etc/netams.conf >/dev/null && echo -n ' NeTAMS'
                 fi
         fi
         ;;
