@@ -1,13 +1,13 @@
 /*************************************************************************
 ***	Authentication, authorization, accounting + firewalling package
-***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
-***	(c) 2002-2005 NeTAMS Development Team
-***	All rights reserved. See 'Copying' file included in distribution
+***	Copyright 1998-2002 Anton Vinokurov <anton@netams.com>
+***	Copyright 2002-2008 NeTAMS Development Team
+***	This code is GPL v3
 ***	For latest version and more info, visit this project web page
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: billing.c,v 1.88.4.6 2005/03/24 19:53:14 anton Exp $ */
+/* $Id: billing.c,v 1.127 2009-08-01 09:23:55 anton Exp $ */
 
 #include "netams.h"
 
@@ -16,92 +16,26 @@ unsigned max_accounts_number=0;
 FeeCounters FC;
 
 //////////////////////////////////////////////////////////////////////////
-SubPlan::SubPlan(oid newid) {
+SubPlan::SubPlan(oid newid): Object() {
 	id=newid;
 	fee=0;
 	spread='M';
 	inc_in=inc_out=0L;
-	flags=0;
+	flags=SPLAN_NONE;
 	pid=0;
 	pay_in=pay_out=0;
 	overdraft_in=overdraft_out=0;
-	next=NULL;
 	connected_plans=0;
 	inc_adjust=0; 
-}
-SubPlan::~SubPlan() {
+	fee_adjust=1; 
 }
 //////////////////////////////////////////////////////////////////////////
-SubPlansL::~SubPlansL() {
-	SubPlan *sp;
-	while(root) {
-		sp=root;
-		root=root->next;
-		delete sp;
-	}
-}
-
-void SubPlansL::Add(SubPlan *s) {
-	SubPlan *t, *d=NULL;
-	for (t=root; t!=NULL; t=t->next){
-		if (t==s) return;
-		d=t;
-	}
-	if (root) d->next=s; else root=s;
-	num_subplans++;
-}
-
-SubPlan *SubPlansL::Add(oid id) {
-	if (this==bSubPlans) return NULL; // we cannot use this method on global subplans list
-	SubPlan *t;
-	for (t=bSubPlans->root; t!=NULL; t=t->next){
-		if (t->id==id) { Add(t); return t; }
-	}
-	return NULL;
-}
-
-void SubPlansL::Delete(SubPlan *s) {
-	SubPlan *t, *d=NULL;
-	for (t=root; t!=NULL; t=t->next){
-		if (t==s) {
-			if (t==root) root=t->next;
-			else d->next=t->next;
-			num_subplans++;
-			break;
-		}
-		d=t;
-	}
-}
-
-void SubPlansL::Delete(oid id) {
-	SubPlan *t;
-	for (t=root; t!=NULL; t=t->next){
-		if (t->id==id) { Delete(t); return; }
-	}
-}
-
-u_char SubPlansL::Check(SubPlan *s) {
-	SubPlan *t;
-	for (t=root; t!=NULL; t=t->next){
-		if (t==s) return 1;
-	}
-	return 0;
-}
-
-SubPlan *SubPlansL::Check(oid id) {
-	SubPlan *t=NULL;
-	for (t=root; t!=NULL; t=t->next){
-		if (t->id==id) break;
-	}
-	return t;
-}
-//////////////////////////////////////////////////////////////////////////
-Plan::Plan(oid newid) {
+Plan::Plan(oid newid): Object() {
 	id=newid;
 	name=description=NULL;
-	next=NULL;
 	root=NULL; 
 	connected_accounts=0;
+	num_subplans=0;
 }
 
 Plan::~Plan() {
@@ -164,159 +98,122 @@ void Plan::SetAccountData(Account *ac) {
 		
 		bd->h.from=bd->d.from=bd->w.from=bd->m.from=FC.mt;
 
-		if(!(sp->flags&UNLIM_IN) && sp->inc_in) {
+		if(!(sp->flags&SPLAN_UNLIM_IN) && sp->inc_in) {
 			bd->m.in = -(long long)sp->inc_in;
+			if (sp->inc_adjust) { //adjust IN bstats accroding time
+				long long tmp = (long long)((double)sp->inc_in*FC.traf_mult);
+				aDebug(DEBUG_BILLING, "Adjusting IN bstats for %s (%06X) from %lld to %lld\n", \
+					name, id, -bd->m.in, tmp);
+				bd->m.in = -tmp;
+			}
 			bd->flags|=BDATA_NEED_SYNC;
 		}
-		if(!(sp->flags&UNLIM_OUT) && sp->inc_out) {
+		if(!(sp->flags&SPLAN_UNLIM_OUT) && sp->inc_out) {
 			bd->m.out = -(long long)sp->inc_out;
+			if (sp->inc_adjust) { //adjust OUT bstats accroding time
+				long long tmp=(long long)((double)sp->inc_out*FC.traf_mult);
+				aDebug(DEBUG_BILLING, "Adjusting OUT bstats for %s (%06X) from %lld to %lld\n", \
+					name, id, -bd->m.out, tmp);
+				bd->m.out = -tmp;
+			}
 			bd->flags|=BDATA_NEED_SYNC;
 		}
 	}
 }
 //////////////////////////////////////////////////////////////////////////////////////////
-PlansL::~PlansL() {
-	Plan *p;
-	while(root) {
-		p=root;
-		root=root->next;
-		delete p;
-	}
+AccountsList::AccountsList(): List() {
+	max_accounts_number_set=max_accounts_number;
+	last_update=0;
 }
 
-void PlansL::Add(Plan *s) {
-	Plan *t, *d=NULL;
-	for (t=root; t!=NULL; t=t->next){
-		if (t==s) return;
-		d=t;
-	}
-	if (root) d->next=s; else root=s;
-	num_plans++;
-}
+Account* AccountsList::Get(char *param){
+	Account *t=NULL;
+	
+	oid id=strtol(param, NULL, 16);
+	t=(Account*)getById(id);
+	if(t) return t;
 
-void PlansL::Delete(Plan *s) {
-	Plan *t, *d=NULL;
-	for (t=root; t!=NULL; t=t->next){
-		if (t==s) {
-			if (t==root) root=t->next;
-			else d->next=t->next;
-			num_plans++;
-			break;
-		}
-		d=t;
-	}
-}
+	netams_rwlock_rdlock(&rwlock);
+	for (t=(Account*)root; t!=NULL; t=(Account*)t->next)
+		if (STREQ(t->name,param)) break;
+	netams_rwlock_unlock(&rwlock);
 
-void PlansL::Delete(oid id) {
-	Plan *t;													  
-	for (t=root; t!=NULL; t=t->next){
-		if (t->id==id) { Delete(t); return; }
-	}
-}
-
-u_char PlansL::Check(Plan *s) {
-	Plan *t;
-	for (t=root; t!=NULL; t=t->next){
-		if (t==s) return 1;
-	}
-	return 0;
-}
-
-Plan *PlansL::Check(oid id) {
-	Plan *t=NULL;
-	for (t=root; t!=NULL; t=t->next){
-		if (t->id==id) break;
-	}
 	return t;
 }
-//////////////////////////////////////////////////////////////////////////////////////////
-AccountsL::AccountsL() {
-	root=NULL; 
-	num_accounts=0; 
-	max_accounts_number_set=max_accounts_number;
-	last_charge=last_update=0;
-}
 
-AccountsL::~AccountsL() {
-	Account *ac;
-
-	while(root) {
-		ac=root;
-		root=root->next;
-		delete ac;
-	}
-}
-
-Account* AccountsL::Check(char *name){
-	Account *t=NULL;
-	for (t=root; t!=NULL; t=t->next){
-		if (t->name && !strcasecmp(t->name,name)) return t;
-	}
-	return NULL;
-}
-
-void AccountsL::Add(Account *s) {
-	Account *t, *d=NULL;
-	for (t=root; t!=NULL; t=t->next){
-		if (t==s) return;
-		d=t;
-	}
-	if (root) d->next=s; else root=s;
-	num_accounts++;
-}
-
-void AccountsL::ChargeFees(){
-	time_t change_t=FC.ht;
+void AccountsList::UpdateAccounts() {
+	FeeCounters *fc=PrepareFeeCounters();
+	time_t hour_change_t	= fc->ht;
+	time_t month_change_t	= fc->mt;
 	
-	if(change_t <= last_charge) return;
+	if(hour_change_t <= last_update) return;
+	
+	billing_data *bd;
 
-	for(Account *ac=root; ac!=NULL; ac=ac->next) 
-		ac->ChargeFee();
+	netams_rwlock_rdlock(&rwlock);
+	for(Account *ac=(Account*)root; ac!=NULL; ac=(Account*)ac->next) {
+		u_char poz=0;
+		for (bSPlist *spl=ac->plan?ac->plan->root:NULL; spl!=NULL; spl=spl->next, poz++){
+			bd=&ac->data[poz];
+			
+			//update periodic counters
+			if(bd->h.from!=fc->ht) {
+				bd->h.in=bd->h.out=0;
+				bd->h.from=fc->ht;
+				if(bd->d.from!=fc->dt){
+					bd->d.in=bd->d.out=0;
+					bd->d.from=fc->dt;
+					if(bd->w.from!=fc->wt){
+						bd->w.in=bd->w.out=0;
+						bd->w.from=fc->wt;
+					}
+				}
+			}
+		}
 
-	last_charge=change_t;
-}
-
-void AccountsL::UpdateAccounts() {
-	time_t change_t=FC.mt;
-
-	if(change_t <= last_update) return;
-
-	for(Account *ac=root; ac!=NULL; ac=ac->next) {
-		if(change_t > ac->blocked && (ac->status&ACCOUNT_BEBLOCKED)) {
+		if(month_change_t > ac->blocked && (ac->status&ACCOUNT_BEBLOCKED)) {
 			aDebug(DEBUG_BILLING, "activate pending blocks for account %s(%06X)\n",ac->name,ac->id);
-			ac->UpdateStatus(AC_BLOCK, change_t);
+			ac->UpdateStatus(AC_BLOCK, month_change_t);
 		}
 		//look into Update() - if nextplan==plan - we only update counters
-		if(change_t > ac->nextplan_ch) {
+		if(month_change_t > ac->nextplan_ch) {
+			if (ac->nextplan_ch!=0) {
+				LogEvent(BILLING, 0, 0, ac->id, "balance NOW %lf", ac->balance);
+			}
 			ac->Update(ac->nextplan);
 		}
+		
+		//Charge Fee
+		ac->ChargeFee();
 	}
-
-	last_update=change_t;
+	last_update=hour_change_t;
+	netams_rwlock_unlock(&rwlock);
 }
 
-void AccountsL::RestoreAccounts() {
+void AccountsList::RestoreAccounts() {
 //	time_t month_t=FC.mt;
 //	time_t now=FC.now;
 //	time_t changed;
 
-	for(Account *ac=root; ac!=NULL; ac=ac->next) {
+	netams_rwlock_rdlock(&rwlock);
+	for(Account *ac=(Account*)root; ac!=NULL; ac=(Account*)ac->next) {
 		if(ac->plan) ac->plan->SetAccountData(ac);
 
 		ac->status&=~ACCOUNT_NEED_SYNC;
 		ac->status&=~ACCOUNT_BDATA_NEED_SYNC;
-	}	
+	}
+	netams_rwlock_unlock(&rwlock);
 }
 //////////////////////////////////////////////////////////////////////////////////////////
 Account::Account(){
 	time_t t=time(NULL);  
 
-	id=newOid(0);
+	id=0;
 	balance=0; 
+	credit_limit=0;
 	blocked=t; created=t; changed=t; 
 	status=ACCOUNT_BLOCKED|ACCOUNT_DENIED; //set initial status
 	description=NULL;
-	next=NULL; 
 	num_units=0;
 	email=password=NULL; 
 	plan=nextplan=NULL; 
@@ -397,13 +294,16 @@ void Account::SetUnitAccountPolicy(NetUnit *u, u_char flag) {
 
 	if(!plan) return; //no need to allocate
 	u->ap=new PdList();
+	char *s = NULL;
+	print_to_string(&s, "BILLING_SETPOLICY");
 	for(bsp=plan->root;bsp!=NULL;bsp=bsp->next) {
-		if(!(p=PolicyL.getPolicyById(bsp->sp->pid))) continue;
-		pd=u->ap->Add(p,bsp->sp->flags);
-		sPConstructReadMessages(u->id, pd);
-		aDebug(DEBUG_BILLING, "  Policy %s(%06X) added to unit %s(%06X) from subplan %u\n", p->name, p->id,
-u->name, u->id, bsp->sp->id);
+		if(!(p=(Policy*)PolicyL->getById(bsp->sp->pid))) continue;
+		pd=u->ap->Add(p,bsp->sp->policy_flags);
+		aDebug(DEBUG_BILLING, "  Policy %s(%06X) added to unit %s(%06X) from subplan %u\n", p->name, p->id, u->name, u->id, bsp->sp->id);
+		print_to_string(&s, " %s %06X", p->name, p->id);
 	}
+	cAccessScriptCall(PASS, u, s);
+	aFree(s);
 	//set Unit flags
 	if(u->ap->IsNetCheckBroken(u)) u->flags |= NETUNIT_BROKEN_ACCT_MF;
 	else u->flags &= ~NETUNIT_BROKEN_ACCT_MF;
@@ -481,8 +381,8 @@ void Account::AccountMessage(Message_Store *msg) {
 		if(sp->pid!=msg->ap) continue;
 
 		//account
-		data[poz].flow.in += msg->data->in;
-		data[poz].flow.out += msg->data->out;
+		data[poz].flow_in += msg->data->in;
+		data[poz].flow_out += msg->data->out;
 		status|=ACCOUNT_BDATA_NEED_SYNC;
 		data[poz].flags|=BDATA_NEED_SYNC;
 	}
@@ -521,7 +421,7 @@ FeeCounters* PrepareFeeCounters() {
 }
 
 void Account::ChargeFee() {
-	
+
 	//do not charge fees from blocked && denied accounts accounts
 	if(status&(ACCOUNT_BLOCKED|ACCOUNT_DENIED)) return;
 	
@@ -530,64 +430,29 @@ void Account::ChargeFee() {
 	SubPlan *sp=NULL;
 	double charge=0;
 	double sp_charge;
-	u_char poz=0;
-	long long tmp;
 	FeeCounters *fc=&FC;
-	billing_data *bd;
 	time_t now=fc->now;
 
-	for (bSPlist *spl=plan->root; spl!=NULL; spl=spl->next, poz++){
+	for (bSPlist *spl=plan->root; spl!=NULL; spl=spl->next){
 		sp=spl->sp;
 		
-		bd=&data[poz];
-
-		if (sp->inc_adjust) {
-			//correct bstats according time
-			tmp=(long long)((double)sp->inc_in*fc->traf_mult);
-			if(bd->m.in < -tmp) {
-				aDebug(DEBUG_BILLING, "Adjusting IN bstats for %s (%06X) from %lld to %lld\n", name, id, -bd->m.in, tmp);
-				bd->m.in=-tmp;
-				status|=ACCOUNT_BDATA_NEED_SYNC;
-				bd->flags|=BDATA_NEED_SYNC;
-			}
-			tmp=(long long)((double)sp->inc_out*fc->traf_mult);
-			if(bd->m.out < -tmp) {
-				aDebug(DEBUG_BILLING, "Adjusting OUT bstats for %s (%06X) from %lld to %lld\n", name, id, -bd->m.out, tmp);
-				bd->m.out=-tmp;
-				status|=ACCOUNT_BDATA_NEED_SYNC;
-				bd->flags|=BDATA_NEED_SYNC;
-			}
-		} // sp->inc_adjust!=0
-		//update periodic counters
-		if(bd->h.from!=fc->ht) {
-			bd->h.in=bd->h.out=0;
-			bd->h.from=fc->ht;
-			if(bd->d.from!=fc->dt){
-				bd->d.in=bd->d.out=0;
-				bd->d.from=fc->dt;
-				if(bd->w.from!=fc->wt){
-					bd->w.in=bd->w.out=0;
-					bd->w.from=fc->wt;
-				}
-			}
-		}
-
-
-
 		if(!sp->fee) continue; // no fee
 		
 		switch(sp->spread) {
 			case 'H':
 				if(fc->ht <= last_fee_ch) continue;
-				sp_charge=sp->fee*fc->hour_mult;
+				if (sp->fee_adjust) sp_charge=sp->fee*fc->hour_mult;
+				else sp_charge=sp->fee;
 				break;
 			case 'D':
 				if(fc->dt <= last_fee_ch) continue;
-				sp_charge=sp->fee*fc->day_mult;
+				if (sp->fee_adjust) sp_charge=sp->fee*fc->day_mult;
+				else sp_charge=sp->fee;
 				break;
 			case 'M':
 				if(fc->mt <= last_fee_ch) continue;
-				sp_charge=sp->fee*fc->month_mult;
+				if (sp->fee_adjust) sp_charge=sp->fee*fc->month_mult;
+				else sp_charge=sp->fee;
 				break;
 			default:
 				sp_charge=0;
@@ -601,8 +466,8 @@ void Account::ChargeFee() {
 	
 	if(charge==0) return; //nothing to charge
 
-	if(balance-charge>=0) {
-		Balance(charge, BAL_REMOVE);
+	if(balance-charge>=credit_limit) {
+		balance-=charge;
 		last_fee_ch=now;
 		LogEvent(BILLING, 0, 0, id, "periodic fee %lf for acc. %s charged", charge, name);
 		aDebug(DEBUG_BILLING, "periodic fee %lf for acc. %s charged\n", charge, name);
@@ -610,11 +475,12 @@ void Account::ChargeFee() {
 		UpdateStatus(AC_DENY, now);
 		aDebug(DEBUG_BILLING, "periodic fee %lf for acc. %s cannot be charged\n", charge, name);
 	}
+	status|=ACCOUNT_NEED_SYNC;
 }
 
 void Account::UpdateStatus(status_action action, time_t now) {
 	NetUnit *u;
-	char *action_name=NULL;
+	const char *action_name=NULL;
 	
 	if (action==AC_DENY) {
 		if(status&ACCOUNT_DENIED) return;
@@ -644,7 +510,6 @@ void Account::UpdateStatus(status_action action, time_t now) {
 				status|=ACCOUNT_NEED_SYNC;
 				aDebug(DEBUG_BILLING, "account: %s(%06X) pending block removed\n", name, id);
 				action_name="UNBEBLOCK";
-				sPvmSend(this, action_name);
 			} 
 			return;
 		}
@@ -655,7 +520,6 @@ void Account::UpdateStatus(status_action action, time_t now) {
 	
 	aDebug(DEBUG_BILLING, "account: %s(%06X) balance: %lf action: %s\n", name, id, balance, action_name);
 	status|=ACCOUNT_NEED_SYNC;
-	sPvmSend(this, action_name);
 
 	if(action==AC_BEBLOCK) return;
 
@@ -684,7 +548,6 @@ void Account::UpdateStatus(status_action action, time_t now) {
 }
 
 void Account::Balance(float amount, balance_action action){
-	double prev=balance;
 	time_t now=FC.now;
 
 	switch (action) {
@@ -701,15 +564,18 @@ void Account::Balance(float amount, balance_action action){
 	
 	//be sure about > or >= and < or <=
 	if(status&ACCOUNT_DENIED) {
-		status&=~ACCOUNT_DENIED;
-		ChargeFee();
-		if(!(status&ACCOUNT_DENIED)) {
-			status|=ACCOUNT_DENIED; //we need for correct UpdateStatus() 
-			UpdateStatus(AC_PASS, now);
+		if (balance >= credit_limit) {
+			status&=~ACCOUNT_DENIED;
+			ChargeFee();
+			if(!(status&ACCOUNT_DENIED)) {
+				status|=ACCOUNT_DENIED; //we need for correct UpdateStatus() 
+				UpdateStatus(AC_PASS, now);
+			}
 		}
-	} else if(prev>=0 && balance<0)
+	} else if(balance<credit_limit) {
 		UpdateStatus(AC_DENY, now);
-		
+	}
+
 	status|=ACCOUNT_NEED_SYNC;
 }
 
@@ -721,7 +587,7 @@ void Account::SyncAccount(FILE *bfile) {
 	for (bUlist *bu=bUroot; bu!=NULL; bu=bu->next)
 		fprintf(bfile, "%06X ", bu->u->id);
 
-	fprintf(bfile, ",%u,%u,%u,%u,%u,%u,%u,%u,%s,%s,%u\n", plan?plan->id:0, plan_ch, nextplan?nextplan->id:0, nextplan_ch, blocked, created, changed, last_fee_ch, email?email:" ", password?password:" ", status);
+	fprintf(bfile, ",%u,%lu,%u,%lu,%lu,%lu,%lu,%lu,%s,%s,%u,%lf\n", plan?plan->id:0, plan_ch, nextplan?nextplan->id:0, nextplan_ch, blocked, created, changed, last_fee_ch, email?email:" ", password?password:" ", status, credit_limit);
 	status&=~ACCOUNT_NEED_SYNC;
 	
 }
@@ -744,7 +610,14 @@ unsigned Account::SyncBdata(FILE *sfile) {
 			
 			bd=&data[poz];
 
-			delta=bd->flow.in;
+			if((sp->flags&SPLAN_SUM) && bd->m.in<0) {
+				//remember bd->m.in==bd->m.in until they preincluded 
+				//e.g. below 0
+				bd->m.in  += bd->flow_out;
+				bd->m.out += bd->flow_in;
+			}
+
+			delta=bd->flow_in;
 			//in
 			bd->m.in+=delta;
 			if(delta && bd->m.in>0) {
@@ -756,7 +629,7 @@ unsigned Account::SyncBdata(FILE *sfile) {
 				bd->d.in+=delta;
 				bd->w.in+=delta;
 
-				if(!(sp->flags&UNLIM_IN)) {
+				if(!(sp->flags&SPLAN_UNLIM_IN)) {
 					charge=sp->overdraft_in*delta;
 					bd->h.pay_in+= charge;
 					bd->d.pay_in+= charge;
@@ -768,19 +641,19 @@ unsigned Account::SyncBdata(FILE *sfile) {
 				}
 			}
 			
-			delta=bd->flow.out;
+			delta=bd->flow_out;
 			//out
 			bd->m.out+=delta;
 			if(delta && bd->m.out>0) {
 				if (bd->m.out < delta) {
-					aDebug(DEBUG_BILLING, "Preincluded in traffic=%llu is over, switching to overdraft\n", sp->inc_out);
+					aDebug(DEBUG_BILLING, "Preincluded out traffic=%llu is over, switching to overdraft\n", sp->inc_out);
 					delta = bd->m.out;
 				}
 				bd->h.out+=delta;
 				bd->d.out+=delta;
 				bd->w.out+=delta;
 
-				if(!(sp->flags&UNLIM_OUT)) {
+				if(!(sp->flags&SPLAN_UNLIM_OUT)) {
 					charge = sp->overdraft_out*delta;
 					bd->h.pay_out+= charge;
 					bd->d.pay_out+= charge;
@@ -793,15 +666,15 @@ unsigned Account::SyncBdata(FILE *sfile) {
 			}
 			
 			if(bd->m.in>0 || bd->m.out>0) {
-				fprintf(sfile, "%c,%u,%u,%u,%lld,%lld,%lf,%lf\n",'H',id,bsp->sp->id,bd->h.from,bd->h.in,bd->h.out,bd->h.pay_in,bd->h.pay_out);
-				fprintf(sfile, "%c,%u,%u,%u,%lld,%lld,%lf,%lf\n",'D',id,bsp->sp->id,bd->d.from,bd->d.in,bd->d.out,bd->d.pay_in,bd->d.pay_out);
-				fprintf(sfile, "%c,%u,%u,%u,%lld,%lld,%lf,%lf\n",'W',id,bsp->sp->id,bd->w.from,bd->w.in,bd->w.out,bd->w.pay_in,bd->w.pay_out);
+				fprintf(sfile, "%c,%u,%u,%lu,%lld,%lld,%lf,%lf\n",'H',id,bsp->sp->id,bd->h.from,bd->h.in,bd->h.out,bd->h.pay_in,bd->h.pay_out);
+				fprintf(sfile, "%c,%u,%u,%lu,%lld,%lld,%lf,%lf\n",'D',id,bsp->sp->id,bd->d.from,bd->d.in,bd->d.out,bd->d.pay_in,bd->d.pay_out);
+				fprintf(sfile, "%c,%u,%u,%lu,%lld,%lld,%lf,%lf\n",'W',id,bsp->sp->id,bd->w.from,bd->w.in,bd->w.out,bd->w.pay_in,bd->w.pay_out);
 				num+=3;
 			}
-			fprintf(sfile, "%c,%u,%u,%u,%lld,%lld,%lf,%lf\n",'M',id,bsp->sp->id,bd->m.from,bd->m.in,bd->m.out,bd->m.pay_in,bd->m.pay_out);
+			fprintf(sfile, "%c,%u,%u,%lu,%lld,%lld,%lf,%lf\n",'M',id,bsp->sp->id,bd->m.from,bd->m.in,bd->m.out,bd->m.pay_in,bd->m.pay_out);
 			num++;		
 
-			bd->flow.in=bd->flow.out=0;
+			bd->flow_in=bd->flow_out=0;
 			data[poz].flags&=~BDATA_NEED_SYNC;
 		}
 	}
@@ -811,7 +684,7 @@ unsigned Account::SyncBdata(FILE *sfile) {
 		balance -= sum;
 		aDebug(DEBUG_BILLING, "  Charged: %lf Balance: %lf\n", charge, balance);
 		status|=ACCOUNT_NEED_SYNC;
-		if(balance <= 0) UpdateStatus(AC_DENY, now);
+		if(balance <= credit_limit) UpdateStatus(AC_DENY, now);
 	}
 
 	status&=~ACCOUNT_BDATA_NEED_SYNC;
