@@ -7,7 +7,7 @@
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: common.c,v 1.193 2009-08-01 09:23:55 anton Exp $ */
+/* $Id: common.c,v 1.196 2009-09-03 13:32:44 jura Exp $ */
 
 #include "netams.h"
 
@@ -593,30 +593,42 @@ void cAccessScriptCall(restrict_type action, NetUnit *u, const char *p){
 	else if (u->type==NETUNIT_USER) {
 		print_to_string(&param, "%s %s", inet_ntop(AF_INET, &((NetUnit_user*)u)->ip, buf, 32), p);
 	}
+	else if (u->type==NETUNIT_NET) {
+		print_to_string(&param, "%s/%u %s",
+				inet_ntop(AF_INET, &((NetUnit_net*)u)->ip, buf, 32),
+				MASK2MLEN(((NetUnit_net*)u)->mask), p);
+	}
 	else {
-		param=set_string(p);
+		aLog(D_WARN, "cAccessScriptCall:[Unit %06X] %s unit type not supported yet\n",
+					u->id, netunit_type_name[u->type]);
+		return;
+//		param=set_string(p);
 	}
 
-	unsigned len = strlen(Processor->access_script) + 15 + strlen(param);
-	char *buffer = (char*)aMalloc(len);
+
+	char *buffer = NULL;
+	const char *action_name;
 	oid oid=u->id;
 
 	switch (action) {
 		case DROP:
-			snprintf(buffer, len, "%s DENY %06X %s", Processor->access_script, oid, param?param:"");
+			action_name="DROP";
+			print_to_string(&buffer, "%s DENY %06X %s", Processor->access_script, oid, param?param:"");
 			break;
 		case PASS:
-			snprintf(buffer, len, "%s ALLOW %06X %s", Processor->access_script, oid, param?param:"");
+			action_name="PASS";
+			print_to_string(&buffer, "%s ALLOW %06X %s", Processor->access_script, oid, param?param:"");
 			break;
 		case PASS_LOCAL:
-			snprintf(buffer, len, "%s LOCAL %06X %s", Processor->access_script, oid, param?param:"");
+			action_name="PASS_LOCAL";
+			print_to_string(&buffer, "%s LOCAL %06X %s", Processor->access_script, oid, param?param:"");
 			break;
 		default:
 			break;
 	}
 
 	i=netams_system(buffer);
-	aLog(D_INFO, "cAccessScriptCall oid %06X action %d system:%d\n", oid, action, i);
+	aLog(D_INFO, "cAccessScriptCall oid %06X action %s system:%d\n", oid, action_name, i);
 	aFree(param);
 	aFree(buffer);
 }
@@ -779,7 +791,7 @@ const char *daysofweek[7] = { "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun" };
 //////////////////////////////////////////////////////////////////////////////////////////
 #ifndef HAVE_BILLING
 #ifndef MAKE_FOR
-char *buildforstring=NULL;
+const char *buildforstring=NULL;
 #endif
 #endif
 //////////////////////////////////////////////////////////////////////////////////////////
