@@ -1,18 +1,22 @@
 /*************************************************************************
 ***	Authentication, authorization, accounting + firewalling package
-***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
-***	(c) 2002-2005 NeTAMS Development Team
-***	All rights reserved. See 'Copying' file included in distribution
+***	Copyright 1998-2002 Anton Vinokurov <anton@netams.com>
+***	Copyright 2002-2008 NeTAMS Development Team
+***	This code is GPL v3
 ***	For latest version and more info, visit this project web page
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: common.h,v 1.41 2005/01/17 13:13:21 jura Exp $ */
+/* $Id: common.h,v 1.63 2009-08-01 09:23:55 anton Exp $ */
 
+//////////////////////////////////////////////////////////////////////////////////////////
+#define KILL            1
+#define RELOAD          2
+#define SHUTDOWN        3
+extern const char *signal_name[];
 //////////////////////////////////////////////////////////////////////////////////////////
 // Logging Management ///////////////////////////////////////////////////////////////////
-void aParse(Connection *conn, char const *fmt, ...);
-void aLog(u_char level, char const *fmt, ...);
+void aLog(u_char level, char const *fmt, ...) __attribute__ ((format (printf, 2, 3)));
 
 #define	D_EMERG	    0x0000  /* for program start/stop records*/
 #define	D_ALERT	    0x0001  /* for threads start/stop, db open/close ? */
@@ -24,29 +28,27 @@ void aLog(u_char level, char const *fmt, ...);
 #define	D_DEBUG	    0x0007  /* all debugging info is on */
 #define	D_DEFAULT_DEBUG	    D_INFO
 
-enum event_type { SYSTEM=0, ACCESS, LOGIN, QUOTA, BILLING };	
- 
+enum event_type { SYSTEM=0, ACCESS, LOGIN, QUOTA, BILLING, RADIUS };	
 //////////////////////////////////////////////////////////////////////////////////////////
-
 typedef unsigned oid;
 oid newOid(oid newid=0);
-
 //////////////////////////////////////////////////////////////////////////////////////////
 // Debugging management
 typedef long long Debug;
+extern Debug debug;
 
 #if defined(DEBUG ) || defined(OLD_GCC)
-void aDebug(u_char d, char const *fmt, ...);
+u_char aDebug(u_char d, char const *fmt, ...);
 #else
 #define aDebug(...)
 #endif
 
 void aDebugInit();
-u_char aDebugAdd(Debug &d, char *str);
-u_char aDebugRm(Debug &d, char *str);
-u_char aDebugIsSet(Debug d, u_char u);
+u_char aDebugAdd(Debug *d, const char *str);
+u_char aDebugRm(Debug *d, const char *str);
+u_char aDebugIsSet(Debug *d, u_char u);
 
-extern char *debug_list[64]; 
+extern const char *debug_list[64]; 
 
 #define DEBUG_NONE		0
 #define	DEBUG_COMMAND		1
@@ -68,7 +70,10 @@ extern char *debug_list[64];
 #define DEBUG_MEMORY		18
 #define DEBUG_POLICY		19
 #define DEBUG_BILLING		20
-#define DEBUG_PVM		21
+#define DEBUG_BW		22
+#define DEBUG_ACLSERVER		23
+#define DEBUG_MUTEX		24
+#define DEBUG_DS_RAW	25
 #define DEBUG_ALL		63
 
 //////////////////////////////////////////////////////////////////////////////////////////
@@ -91,11 +96,12 @@ void PrepareTimeCounters(struct time_counters *tc, time_t t=time(NULL));
 void FillTimeCounters(policy_data *np,struct time_counters *tc);
 time_t aGetActual(char prefix, time_t t=time(NULL));
 //////////////////////////////////////////////////////////////////////////////////////////
-void cHelp(Connection *conn, u_char no_flag, char *str);
 void aShowVersion (FILE*);
 void print_to_string(char **string, char const *fmt, ...);
-char* set_string(char *string);
-void cExec(char *cmd, char **buffer);
+char* set_string(const char *string);
+char* cExec(const char *cmd);
+int aCommand(const char *cmd, int mode, FILE *f=NULL);
+int netams_system(const char* command);
 //////////////////////////////////////////////////////////////////////////////////////////
 u_char mask2mlen(in_addr *addr);
 /* taken from http://groups.google.com/groups?hl=ru&lr=&threadm=cl3a87%24pa5%244%40wagner.wagner
@@ -103,13 +109,11 @@ u_char mask2mlen(in_addr *addr);
 25D0%25B5%25D1%2582%25D0%25B8%2Bgroup:*ru*unix.prog%26hl%3Dru%26lr%3D%26scoring%3Dr%26selm%3Dcl3
 a87%2524pa5%25244%2540wagner.wagner.home%26rnum%3D1  */
 #define MLEN2MASK(mlen) ((mlen)?(((in_addr_t)~(uint32_t)0) << (32 - (mlen))):0)
-#define MASK2MLEN(mask) mask2mlen(mask)
+#define MASK2MLEN(mask) mask2mlen(&mask)
 //////////////////////////////////////////////////////////////////////////////////////////
 void aOidsInit();
-int cNewOid(Connection *c, char *type);
-int cShowPerf(Connection *conn, char *param[]);
-int cShowSystemHealth(Connection *conn);
-u_char getAddr(char *str, in_addr *addr, in_addr *mask=NULL);
-int getdayofweek(char *str);
-extern char *daysofweek[7];
+int aShowSystemHealth(struct cli_def *cli, u_char is_html=0);
+int getAddr(char *str, in_addr *addr, in_addr *mask=NULL);
+int getdayofweek(const char *str);
+extern const char *daysofweek[7];
 //////////////////////////////////////////////////////////////////////////////////////////
