@@ -7,7 +7,7 @@
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: ds_ipfw.c,v 1.43 2008-02-23 08:35:02 anton Exp $ */
+/* $Id: ds_ipfw.c,v 1.44 2009-12-19 17:25:50 anton Exp $ */
 
 #ifdef FREEBSD
 #include "netams.h"
@@ -30,14 +30,15 @@ void ds_ipfw(Service_DS *ds) {
         sin.sin_port = htons(ds->port);
 
 
-        if ((socketid = socket(PF_INET, SOCK_RAW, IPPROTO_DIVERT)) < 0) {
+        if ((socketid = socket(PF_INET, SOCK_RAW, IPPROTO_DIVERT)) < 0) 
 		aLog(D_ERR, "divert socket\n");
-		goto END;
-	}
-        if (bind(socketid, (struct sockaddr *)&(sin), sizeof(sin))) {
-		aLog(D_ERR, "bind divert socket: %u\n", socketid);
-		goto END;
-	}
+	else {
+		
+	
+        if (bind(socketid, (struct sockaddr *)&(sin), sizeof(sin))) 
+        	aLog(D_ERR, "bind divert socket: %u\n", socketid);
+	else {
+	
 	SET_POLL(socketid);
 
 	pthread_cleanup_push(ds_ipfw_cancel, (void*) &socketid);
@@ -80,8 +81,11 @@ void ds_ipfw(Service_DS *ds) {
 #endif
 		ds->Measure(&start, len);
 	}
-END:
+	
+	} // socket
+	} // bind
 	pthread_cleanup_pop(1);
+	
 }
 /////////////////////////////////////////////////////////////////////////////////////
 void ds_ipfw_cancel(void *ptr) {
