@@ -1,17 +1,19 @@
 /*************************************************************************
 ***	Authentication, authorization, accounting + firewalling package
-***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
-***	(c) 2002-2005 NeTAMS Development Team
-***	All rights reserved. See 'Copying' file included in distribution
+***	Copyright 1998-2002 Anton Vinokurov <anton@netams.com>
+***	Copyright 2002-2008 NeTAMS Development Team
+***	This code is GPL v3
 ***	For latest version and more info, visit this project web page
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: flowprobe.c,v 1.42 2005/01/17 13:13:21 jura Exp $ */
+/* $Id: flowprobe.c,v 1.48 2009-08-01 09:23:55 anton Exp $ */
 
 #ifndef NOPCAP
 #include "config.h"
-#include "lib.h"
+extern "C" {
+	#include "../lib/lib.h"
+}
 
 typedef void ServiceDS_cfg;
 #include "ds_libpcap.h"
@@ -142,7 +144,7 @@ void PrintUsage(FILE *);
 class NetFlow *Flow;
 
 char *destination=NULL;
-char *dst_host=NULL;
+const char *dst_host=NULL;
 unsigned short dst_port=20001;
 
 extern u_char debug, quiet;
@@ -216,6 +218,8 @@ int main(int argc, char **argv){
 	}
 
 	signal(SIGINT, (sig_t)termination);
+	signal(SIGSTOP, (sig_t)termination);
+	signal(SIGTERM, (sig_t)termination);
 
 	pcap = pcap_open_live(interface, MAX_PKT_SIZE, promisc, 1000, errbuf);
         if (!pcap) {
