@@ -7,7 +7,7 @@
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: html.c,v 1.109 2009-08-01 09:23:55 anton Exp $ */
+/* $Id: html.c,v 1.111 2010-01-17 14:15:57 anton Exp $ */
 
 #include "netams.h"
 
@@ -681,21 +681,22 @@ int sHSafeMkdir(const char *temp){  // this equal to  mkdir -p
 //////////////////////////////////////////////////////////////////////////////////////////
 void sHPrintHeader(FILE *f, const char *title, const char *back, const char *logopath){
 
-	fprintf(f, "<html><head><title>NeTAMS - %s</title><META http-equiv=\"Pragma\" content=\"no-cache\"><META http-equiv=\"Expires\" content=\"-1\"><META http-equiv=\"Cache-Control\" content=\"no-cache\"><META http-equiv=\"Content-type\" content=\"text/html; charset=%s\">\n</HEAD> \n\
-		<body bgcolor=white marginheight=0 leftmargin=0 topmargin=0 marginwidth=0> \n\
+	fprintf(f, "<html><head><title>NeTAMS - %s</title><META http-equiv=\"Pragma\" content=\"no-cache\"><META http-equiv=\"Expires\" content=\"-1\"><META http-equiv=\"Cache-Control\" content=\"no-cache\"><META http-equiv=\"Content-type\" content=\"text/html; charset=%s\">\n \
+		<script type=\"text/javascript\" src=\"%s/netams.js\"></script> \n\
+		</HEAD><body bgcolor=white marginheight=0 leftmargin=0 topmargin=0 marginwidth=0> \n\
 		<table width=100%% align=left border=0 cellpadding=5 cellspacing=0> \n\
 		<tr bgcolor='white' align=left><td width=376> \n\
 		<A href=\"http://www.netams.com/index.html\"> \n\
 		<img src=\"%s/images/logo_sm.jpg\" width=376 height=60 border=0></a></td> \n\
 		<td valign=middle align=left width=80%%> %s	<a href=\"http://www.netams.com/index.html\"><b>NeTAMS</b></a><br> \
-		<b>%s: </b>", title, language_charset_str[lang], logopath?logopath:"../../../../..", Locale(LC_H_REPORT), Locale(LC_H_SW_VERSION));
+		<b>%s: </b>", title, language_charset_str[lang], logopath?logopath:"../../../../..", logopath?logopath:"../../../../..", Locale(LC_H_REPORT), Locale(LC_H_SW_VERSION));
 	fprintf(f, "%s<br>\n", SHOW_VERSION);
 
 	char buff[32];
 	fprintf(f, "<b>%s:</b> %s<br> </td></tr> <tr bgcolor='#ccccff'><td colspan=2> \n\
 		<h3>%s", Locale(LC_H_CR_TIME), timeU2T(time(NULL), buff), title);
 	if (back)
-		fprintf(f, "<font face=monospace size=-1><b> [<a href=\"%s\"> <- %s</a> ]</b></font>",
+		fprintf(f, "<font face=monospace size=-1><b> [<a href=\"%s\"> <- %s</a> ]&nbsp;[ <a href=\"#\" id='expandall' style='e:1' onclick='expandall();'>expand all</a> ]</b></font>",
 			back, Locale(LC_H_BACK));
 	fprintf(f, "</h3></td></tr><tr><td colspan=2>\n");
 }
@@ -776,7 +777,7 @@ void sHPrintUnitsTree(FILE *f, Service_Html *cfg, NetUnit *u, u_char isadmintool
 
 	if (u->type==NETUNIT_GROUP) {
 		sHPrintTHeader(f);
-		fprintf(f, "<tr bgcolor='#000000'><td colspan=11 height=2> </td></tr><tr bgcolor='#ffffcc'><td colspan=11>");
+		fprintf(f, "<tr bgcolor='#000000'><td colspan=11 height=2> </td></tr><tr bgcolor='#ffffcc' id='%06X'><td colspan=11>", u->id);
 		if (!is_zero_counters && *tmp) {
 		    fprintf(f, "<A href=\"../../../../../cgi-bin/netams_graph.cgi?oid=%06X&policy=%s&year=%u&month=%u&day=%u\">D</a>", u->id, tmp, tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday);
 		    fprintf(f, "<A href=\"../../../../../cgi-bin/netams_graph.cgi?oid=%06X&policy=%s&year=%u&week=d2w-%u-%u-%u\">W</a>", u->id, tmp, tm.tm_year+1900, tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday);
@@ -796,16 +797,16 @@ void sHPrintUnitsTree(FILE *f, Service_Html *cfg, NetUnit *u, u_char isadmintool
 		} else
 		    *tmp=0;
 
-		fprintf(f, "GROUP: <b>%s</b> <font size=-1>oid %06X</font></td></tr>\n", u->name?u->name:"", u->id);
+		fprintf(f, "GROUP: <b>%s</b> <font size=-1>oid %06X</font>&nbsp;<a href=\"#\" id='%06X_c' onclick=\"expand_children('%06X')\">expand children</a>&nbsp;<a href=\"#\" id='%06X_p' onclick=\"expand_policies('%06X')\">expand policies</a></td></tr>\n", u->name?u->name:"", u->id, u->id, u->id, u->id, u->id);
 		sHPrintUnitST(f, u, *tmp?tmp:NULL);
-		fprintf(f, "<tr bgcolor='#ccffcc'><td colspan=11 align=right>\n");
+		fprintf(f, "<tr bgcolor='#ccffcc' id='%06X_child' style='display:none'><td colspan=11 align=right>\n", u->id);
 
 		sHPrintUnitsRoot(f, cfg, u);
 
 		fprintf(f, "</td></tr>\n");
 		fprintf(f, "</table><br>\n");
     } else {
-		fprintf(f, "<tr bgcolor='#ffffcc'><td colspan=11>");
+		fprintf(f, "<tr bgcolor='#ffffcc' id='%06X'><td colspan=11>", u->id);
 		if (isadmintool) fprintf(f, "<A href=\"../../../../../cgi-bin/admintool.cgi?oid=%06X\"><img src=\"../../../../../images/admintool-logo.gif\" width=20 height=20 border=1 align=absmiddle></a> ", u->id);
 		if (!is_zero_counters && *tmp) {
 		    fprintf(f, "<A href=\"../../../../../cgi-bin/netams_graph.cgi?oid=%06X&policy=%s&year=%u&month=%u&day=%u\">D</a>", u->id, tmp, tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday);
@@ -851,7 +852,7 @@ void sHPrintUnitsTree(FILE *f, Service_Html *cfg, NetUnit *u, u_char isadmintool
 		if (u->description) fprintf(f, "&nbsp;&nbsp;&nbsp;<font color=navy>%s</font>", u->description);
 
 		if (is_zero_counters) fprintf(f, "&nbsp;&nbsp;&nbsp;<font color=navy>(%s)</font>", Locale(LC_H_ALL_ZERO));
-  		fprintf(f, "</td></tr>\n\n");
+  		fprintf(f, "&nbsp;<a href=\"#\" id='%06X_p' onclick=\"expand_policies('%06X')\">expand policies</a></td></tr>\n\n", u->id, u->id);
 		if (!is_zero_counters) sHPrintUnitST(f, u, NULL);
 	}
 }
@@ -864,7 +865,7 @@ void sHPrintUnitST(FILE *f, NetUnit *u, const char *unit_list){
 	netams_rwlock_rdlock(&u->ap->rwlock);
 	for (cpd=u->ap->root; cpd!=NULL; cpd=cpd->next) {
 		if (cpd->policy->hidden) continue;
-		fprintf(f, "<tr align=right>");
+		fprintf(f, "<tr align=right style=\"display: none;\" id='%06X_pol'>", u->id);
 		if (unit_list) {
 		    fprintf(f, "<td align=left valign=middle bgcolor='#ffffaa'>");
 		    fprintf(f, "<A href=\"../../../../../cgi-bin/netams_graph.cgi?oid=%s&policy=%06X&year=%u&month=%u&day=%u&oids=unit\">D</a>", unit_list, cpd->policy->id, tm.tm_year+1900, tm.tm_mon+1, tm.tm_mday);
