@@ -1,18 +1,19 @@
 /*************************************************************************
 ***	Authentication, authorization, accounting + firewalling package
-***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
-***	(c) 2002-2005 NeTAMS Development Team
-***	All rights reserved. See 'Copying' file included in distribution
+***	Copyright 1998-2002 Anton Vinokurov <anton@netams.com>
+***	Copyright 2002-2008 NeTAMS Development Team
+***	This code is GPL v3
 ***	For latest version and more info, visit this project web page
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: iptree.c,v 1.49 2005/01/17 13:13:21 jura Exp $ */
+/* $Id: iptree.c,v 1.77 2008-02-23 08:35:03 anton Exp $ */
 
 #include "netams.h"
+#include "flow.h"
+#include "iptree.h"
 
 /////////////////////////////////////////////////////////////////////////////////////////////
-void DSacct(NetUnit *u, IPFlowStat *flow, match mf);
 /////////////////////////////////////////////////////////////////////////////////////////////
 IPTree::IPTree(u_char id) {
 	nodes=1;
@@ -20,23 +21,21 @@ IPTree::IPTree(u_char id) {
 	unodes=0;
 	instance=id;
 	root=(IPTree_node*)aMalloc(sizeof(IPTree_node));
-	rwlock=(pthread_rwlock_t*)aMalloc(sizeof (pthread_rwlock_t));
-	pthread_rwlock_init(rwlock, NULL);
+	netams_rwlock_init(&rwlock, NULL);
 	aLog(D_INFO, "IP tree created\n");
 }
 /////////////////////////////////////////////////////////////////////////////////////////////
 IPTree::~IPTree() {
 	freetree(root);
 	aFree(root);
-	pthread_rwlock_destroy(rwlock);
-        aFree(rwlock);
+	netams_rwlock_destroy(&rwlock);
 	aLog(D_INFO, "IP tree destroyed\n");
 }
 /////////////////////////////////////////////////////////////////////////////////////////////
 void IPTree::FreeTree(IPTree_node *node) {
-        pthread_rwlock_wrlock(rwlock);
+        netams_rwlock_wrlock(&rwlock);
 	freetree(node);
-        pthread_rwlock_unlock(rwlock);
+        netams_rwlock_unlock(&rwlock);
 }
 /////////////////////////////////////////////////////////////////////////////////////////////
 void IPTree::freetree(IPTree_node *node) {
@@ -47,12 +46,11 @@ void IPTree::freetree(IPTree_node *node) {
                 node->next=NULL;
                 dlink--;
 	}
-	dsUlist *tmp,*dsu;
+	dsUlist *dsu;
 	dsu=(dsUlist*)(node->ptr);
-	while(dsu) {
-		tmp=dsu;
-		dsu=dsu->next;
-		aFree(tmp);
+	if(dsu) {
+		ELIST_FREE(dsu->list);
+		aFree(dsu);
 		unodes--;
 	}
 	if(node != root) {
@@ -93,17 +91,17 @@ void IPTree::addr2tree(unsigned addr,u_char mask,NetUnit *u,u_char flag) {
 
 	if(!addr && mask==32) return;
 
-	pthread_rwlock_wrlock(rwlock);
+	netams_rwlock_wrlock(&rwlock);
         
 	if(mask==0) {
 		add2node(root,u,flag);
-		pthread_rwlock_unlock(rwlock);
+		netams_rwlock_unlock(&rwlock);
 		return;
 	}
 	
 //	struct in_addr ia;
 //	ia.s_addr=addr;
-//	aLog(D_INFO,"IPTree: working with %s /%u action=%u(1-add,0-remove)\n",inet_ntoa(ia),mask,flag);
+//	aLog(D_INFO,"IPTree: working with %s /%u action=%u(1-add,0-remove)\n",inet_ ntoa(ia),mask,flag);
 
         for (u_char i=0;i<4;i++) {
                 num=(u_char)(((char*)&addr)[i]);
@@ -147,46 +145,32 @@ void IPTree::addr2tree(unsigned addr,u_char mask,NetUnit *u,u_char flag) {
 	}
 	if(!flag) cleanup(root);
 	
-	pthread_rwlock_unlock(rwlock);
+	netams_rwlock_unlock(&rwlock);
 }
 
-void IPTree::add2node(IPTree_node *node,NetUnit *u,u_char flag) {//1-add 0-remove
+void IPTree::add2node(IPTree_node *node, NetUnit *u,u_char flag) {//1-add 0-remove
 	if(!node) return; //just in case
-	dsUlist *p=NULL;
 	dsUlist *dsu=(dsUlist*)(node->ptr);
-       	for(;dsu!=NULL;dsu=dsu->next) {
-		if(dsu->u==u) break;
-		p=dsu;
-	}
 
 	if(flag) {//add
-		//if dsu!=NULL this is update
-		if(!dsu){
-       	 		dsu=(dsUlist*)aMalloc(sizeof(dsUlist));
-                	dsu->u=u;
-			unodes++;
-			dsu->next=(dsUlist*)(node->ptr);
-               		node->ptr=(void*)dsu;
+			if(!dsu) {
+				dsu = (dsUlist*)aMalloc(sizeof(dsUlist));
+				node->ptr = dsu;
+				unodes++;
+			}
+      		ELIST_ADD(dsu->list, u);
+	} else {//remove
+		ELIST_REMOVE(dsu->list, u);
+		if(ELIST_IS_EMPTY(dsu->list)) { //empty
+			ELIST_FREE(dsu->list);
+			aFree(dsu);
+			unodes--;
+			node->ptr=NULL;
 		}
-
-		//add bw data (bwd)
-		if(u->bw) dsu->bwd = new BWdata(u->bw);
-		else dsu->bwd=NULL;
-      	} else {//remove
-		if(!dsu) return; //nothing to remove
-		if(dsu==(dsUlist*)(node->ptr))
-			node->ptr=(void*)dsu->next;
-                else 
-			p->next=dsu->next;
-		//free bw data
-		if(dsu->bwd) delete dsu->bwd;
-
-                aFree(dsu);
-		unodes--;
-         }
+	}
 }
 /////////////////////////////////////////////////////////////////////
-dsUlist* IPTree::checktree(IPFlowStat *flow, u_char check) {
+dsUlist* IPTree::checktree(Flow *list, u_char check) {
         u_char num;
         IPTree_node *node=root;
 	dsUlist *dsu;
@@ -195,13 +179,18 @@ dsUlist* IPTree::checktree(IPFlowStat *flow, u_char check) {
 	dsUlist *start=(dsUlist*)(root->ptr);
 	u_char *src,*dst;
 	u_char i;
-	
+	const union attribute_value	*value;
+
+	// some checking
+	if((value = list->get(ATTR_IPV4_INFO)) == 0)
+			return start;
+
 	if(start) start->mf[check]=MATCH_BOTH; // it's true for 0/0
 	
-	src=(u_char*)((char*)&flow->srcaddr.s_addr);
-	dst=(u_char*)((char*)&flow->dstaddr.s_addr);
-//	aLog(D_INFO,"src=%s %s\n",inet_ntoa(packet->ip_src),binary(src));
-//	aLog(D_INFO,"dst=%s %s\n",inet_ntoa(packet->ip_dst),binary(dst));
+	src=(u_char*)((char*)&value->ipv4_info.ip_src.s_addr);
+	dst=(u_char*)((char*)&value->ipv4_info.ip_dst.s_addr);
+//	aLog(D_INFO,"src=%s %s\n",inet_ ntoa(packet->ip_src),binary(src));
+//	aLog(D_INFO,"dst=%s %s\n",inet_ ntoa(packet->ip_dst),binary(dst));
 
 //	showtree(root,0,0);
 	
@@ -257,7 +246,8 @@ void IPTree::showtree(IPTree_node *node,unsigned addr,u_char depth) {
         if(node->ptr) {
                 struct in_addr ia;
                 ia.s_addr=addr;
-                aLog(D_INFO,"%s /%u\n", inet_ntoa(ia), depth<<3);
+		char buffer[32];
+                aLog(D_INFO,"%s /%u\n", inet_ntop(AF_INET, &ia, buffer, 32), depth<<3);
         } else {
 		if(node->next)
                 	for(i=0;i<256;i++)
@@ -265,59 +255,57 @@ void IPTree::showtree(IPTree_node *node,unsigned addr,u_char depth) {
         }
 }
 /////////////////////////////////////////////////////////////////////////////////////////////
-void IPTree::ACCT(IPFlowStat *flow) {
-        NetUnit *u, *up;
-        dsUlist *start,*dsulist,*dsu;
+void IPTree::ACCT(Flow *flow) {
+        NetUnit *u;
+        dsUlist *dsu;
         match mf;
         oid au_parent_id=0;
         match auto_unit_mf, units_mf;
+	const union attribute_value	*value;
 	
 	auto_unit_mf=units_mf=MATCH_NONE;
 
-        pthread_rwlock_rdlock(rwlock);
+        netams_rwlock_rdlock(&rwlock);
 
         //here we organize units that matches packet
-        start=checktree(flow,CHECK_ACCT);
+        dsu=checktree(flow,CHECK_ACCT);
+        
+	for(;dsu!=NULL;dsu=dsu->link[CHECK_ACCT]) {
+       		mf=dsu->mf[CHECK_ACCT];
+            
+            	ELIST_FOR_EACH(dsu->list, u) {
+			//check packet might be processed incorrectly due to wrong match flag
+			if(u->flags&NETUNIT_BROKEN_ACCT_MF) {
+                		match mmf=u->Check(flow);
+		    		if(mmf!=mf && mf==MATCH_DST)
+					continue;
+				mf=mmf;
+			} else
+				mf=dsu->mf[CHECK_ACCT];
 
-        for(dsulist=start;dsulist!=NULL;dsulist=dsulist->link[CHECK_ACCT]) {
-                mf=dsulist->mf[CHECK_ACCT];
-                for(dsu=dsulist;dsu!=NULL;dsu=dsu->next) {
-                        u=dsu->u;
-
-                        //check packet might be processed incorrectly due to wrong match flag
-                        if(u->flags&NETUNIT_BROKEN_ACCT_MF) {
-                                match mmf=u->Check(flow);
-                                if(mmf!=mf && mf==MATCH_DST) continue;
-                                mf=mmf;
-                        } else
-                                mf=dsulist->mf[CHECK_ACCT];
-
-                        // now check the acct policy for this packet, and update counters
-                        // for this unit itself
-                        DSacct(u, flow, mf);
+                	// now check the acct policy for this packet, and update counters
+                	DSacct(u, flow, mf);
 			
 			//this is part of auto-units
-                        if(u->type==NETUNIT_NET && ((NetUnit_net*)u)->auto_units_id) {
+			if(u->type==NETUNIT_NET && ((NetUnit_net*)u)->auto_units_id) {
 				au_parent_id=u->id;
 				auto_unit_mf=mf;
-                        } else if(u->type==NETUNIT_HOST || u->type==NETUNIT_USER) {
+			} 
+			else if(u->type==NETUNIT_HOST || u->type==NETUNIT_USER) {
 				units_mf|=mf; //unit already exist
 			}
-
-                        // and for all parents
-                        for (up=u->parent; up!=NULL; up=up->parent)
-                                if (up->checkDSList(instance)) DSacct(up, flow, mf);
-                }
-                dsulist->mf[CHECK_ACCT]=MATCH_NONE;
+            	}
+        	dsu->mf[CHECK_ACCT]=MATCH_NONE;
         }
+        netams_rwlock_unlock(&rwlock);
 
-        pthread_rwlock_unlock(rwlock);
-	
-	if((auto_unit_mf&=~units_mf)) {
-		if(auto_unit_mf&MATCH_SRC) CreateAutoUnit(au_parent_id, flow->srcaddr);
-		if(auto_unit_mf&MATCH_DST) CreateAutoUnit(au_parent_id, flow->dstaddr);
+	value = flow->get(ATTR_IPV4_INFO);	
+	if((auto_unit_mf&=~units_mf) && value != 0) {
+		if(auto_unit_mf&MATCH_SRC) CreateAutoUnit(au_parent_id, value->ipv4_info.ip_src);
+		if(auto_unit_mf&MATCH_DST) CreateAutoUnit(au_parent_id, value->ipv4_info.ip_dst);
 	}
 }
+
 /////////////////////////////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////////////////////////
 // definitions for PrefixTree
@@ -347,12 +335,12 @@ void PrefixTree::freetree(IPTree_node *node) {
 	}
 }
 /////////////////////////////////////////////////////////////////////////////////////////////
-void PrefixTree::prefix2tree(prefix *p) {
+void PrefixTree::prefix2tree(net_prefix *p) {
         IPTree_node *tmp;
 	IPTree_node *node=root;
         u_short num;
         int range;
-	prefix *old;
+	net_prefix *old;
         u_char mask=p->mask;
         u_char* addr=(u_char*)((char*)&p->network.s_addr);
 
@@ -374,15 +362,15 @@ void PrefixTree::prefix2tree(prefix *p) {
                                         tmp=(IPTree_node*)aMalloc(sizeof(IPTree_node));
                                         node->next[num]=tmp;
 					nodes++;
-                                } else if((old=(prefix*)node->next[num]->ptr)) {
+                                } else if((old=(net_prefix*)node->next[num]->ptr)) {
 					if(old->mask > p->mask) 
 						goto NEXT; // previous has higer mask
 					else if (old->mask == p->mask &&
 						old->flags!=p->flags) {
 						char buf1[32], buf2[32];
-						strcpy(buf1, inet_ntoa(p->network));
-						strcpy(buf2, inet_ntoa(old->network));
-						aLog(D_WARN, "Prefix %s%s%/%u conflicts with %s%s/%u, using 1st\n",
+						inet_ntop(AF_INET, &(p->network), buf1, 32);
+						inet_ntop(AF_INET, &(old->network), buf2, 32);
+						aLog(D_WARN, "Prefix %s%s/%u conflicts with %s%s/%u, using 1st\n",
 							(p->flags&PREFIX_INVERTED)?"!":"",buf1,p->mask,
 							(old->flags&PREFIX_INVERTED)?"!":"",buf2,old->mask);
 						return;
@@ -420,7 +408,7 @@ u_char PrefixTree::checktree(unsigned addr) {
         }
 	
 	if(node->ptr) {
-		prefix *p=(prefix*)node->ptr;
+		net_prefix *p=(net_prefix*)node->ptr;
 		p->match++;
 		if(!(p->flags&PREFIX_INVERTED)) return 1;
 	}
@@ -430,38 +418,56 @@ u_char PrefixTree::checktree(unsigned addr) {
 /////////////////////////////////////////////////////////////////////////////////////////////
 // below needed for FW and ACCTcheck
 /////////////////////////////////////////////////////////////////////////////////////////////
-void DSacct(NetUnit *u, IPFlowStat *flow, match mf){
+void IPTree::DSacct(NetUnit *u, Flow *flow, match mf){
 	if(!u->ap) return;
 
 	policy_data *cpd;
 	u_char monitor=0;
-
-	//we ensure nobody reads or writes through this unit 
-	pthread_rwlock_wrlock(u->ap->rwlock);
-
+//	u_char monitor_because_of_layer7=0;
+	
 	aDebug(DEBUG_DS_IP, " policy_check for unit: %s (oid=%06X) ---\n", u->name, u->id);
+	
+	//we ensure nobody reads or writes through this unit 
+	netams_rwlock_wrlock(&u->ap->rwlock);
 	for (cpd=u->ap->root; cpd!=NULL; cpd=cpd->next) {
 		cpd->check++;
 		if(cpd->policy->Check(flow, mf)) {
-			if (!(cpd->flags&POLICY_FLAG_INV)) {
+			if (!(cpd->policy_flags&POLICY_FLAG_INV)) {
 				cpd->match++;
 				monitor=1;
 				PolicyDataUpdate(mf, cpd, flow);
-				if (cpd->flags&POLICY_FLAG_BRK) break;
+//				if (cpd->policy->target.target_type&PT_LAYER7) {
+//					struct layer7_info_value *layer7_info=(struct layer7_info_value*)flow->get(ATTR_LAYER7_INFO);
+//					if (layer7_info) 
+//						if (layer7_info->value) monitor_because_of_layer7=1;
+//				}
+				if (cpd->policy_flags&POLICY_FLAG_BRK) break;
 			}
 		} else {
-			if (cpd->flags&POLICY_FLAG_INV) {
+			if (cpd->policy_flags&POLICY_FLAG_INV) {
 				cpd->match++;
 				monitor=1;
 				PolicyDataUpdate(mf, cpd, flow);
-				if (cpd->flags&POLICY_FLAG_BRK) break;
+				if (cpd->policy_flags&POLICY_FLAG_BRK) break;
 			}
 		}
 	}
-	pthread_rwlock_unlock(u->ap->rwlock);
+	netams_rwlock_unlock(&u->ap->rwlock);
+
+	// and for all parents
+	NetUnit_group *gr;
+	ELIST_FOR_EACH(u->parents, gr) {
+		if (gr->checkDSList(instance))
+			DSacct(gr, flow, mf);
+	}
 	
-	if (u->monitor && monitor) aMonitor(u, flow);
-//	if (u->monitor) aMonitor(u, flow);
+//i dislike to monitor because of layer7
+//better to monitor via policy - hope wthis will be done
+	if(monitor && !ELIST_IS_EMPTY(u->monitors)) {
+		Service *s;
+		ELIST_FOR_EACH(u->monitors, s)
+			((Service_Monitor_Interface*)s)->aMonitor(u, flow);
+	}
 }
 /////////////////////////////////////////////////////////////////////////////////////////////
 /////////////////////////////////////////////////////////////////////////////////////////////
