@@ -1,19 +1,29 @@
 /*************************************************************************
 ***	Authentication, authorization, accounting + firewalling package
-***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
-***	(c) 2002-2005 NeTAMS Development Team
-***	All rights reserved. See 'Copying' file included in distribution
+***	Copyright 1998-2002 Anton Vinokurov <anton@netams.com>
+***	Copyright 2002-2008 NeTAMS Development Team
+***	This code is GPL v3
 ***	For latest version and more info, visit this project web page
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: iptree.h,v 1.16 2005/01/17 13:13:21 jura Exp $ */
+/* $Id: iptree.h,v 1.27 2008-02-23 08:35:02 anton Exp $ */
 
 typedef struct IPTree_node {
 	IPTree_node *up;
 	IPTree_node **next;
 	void *ptr;
-} IPTree_node;
+} __attribute__((packed)) IPTree_node;
+
+// this using un datasource service as unit list representation
+typedef struct dsUlist {
+	NetUnit **list;	//ELIST
+
+	//this used to organize matched units in list
+	//for 0-account, 1-forward check
+	dsUlist *link[2];
+	match mf[2];
+} __attribute__((packed)) dsUlist;
 /////////////////////////////////////////////////////////////////////////////////////
 class IPTree {
 	public:
@@ -21,7 +31,7 @@ class IPTree {
 		unsigned long dlink; //number of *IPTree_node[256] allocated;
 		unsigned long unodes; //number of dsUlist allocated
 		IPTree_node *root;
-		pthread_rwlock_t *rwlock;
+		pthread_rwlock_t rwlock;
 		u_char instance;
 
 		IPTree(u_char instance);
@@ -31,10 +41,11 @@ class IPTree {
 		u_char cleanup(IPTree_node *node);
 		void add2node(IPTree_node *node,NetUnit *u,u_char flag);
 		void addr2tree(unsigned addr,u_char mask,NetUnit *u,u_char flag);
-		dsUlist *checktree(IPFlowStat *flow, u_char check);
+		dsUlist *checktree(Flow *flow, u_char check);
 		void showtree(IPTree_node *node,unsigned addr,u_char depth);
 		
-		void ACCT(IPFlowStat *flow);
+		void ACCT(Flow *flow);
+		void DSacct(NetUnit *u, Flow *flow, match mf);
 };
 /////////////////////////////////////////////////////////////////////////////////////
 class PrefixTree {
@@ -46,7 +57,7 @@ class PrefixTree {
                 PrefixTree();
                 ~PrefixTree();
                 void freetree(IPTree_node *node); //this makes recursion and real free
-                void prefix2tree(prefix *p);
+                void prefix2tree(net_prefix *p);
                 u_char checktree(unsigned addr);
 };
 /////////////////////////////////////////////////////////////////////////////////////
