@@ -1,13 +1,13 @@
 /*************************************************************************
 ***	Authentication, authorization, accounting + firewalling package
-***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
-***	(c) 2002-2005 NeTAMS Development Team
-***	All rights reserved. See 'Copying' file included in distribution
+***	Copyright 1998-2002 Anton Vinokurov <anton@netams.com>
+***	Copyright 2002-2008 NeTAMS Development Team
+***	This code is GPL v3
 ***	For latest version and more info, visit this project web page
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: messages_fifo.h,v 1.17 2005/01/17 13:13:21 jura Exp $ */
+/* $Id: messages_fifo.h,v 1.32 2008-02-23 08:35:02 anton Exp $ */
 
 
 //////////////////////////////////////////////////////////////////////////
@@ -15,25 +15,24 @@
 #define NUM_MSG_TYPES	6
 
 typedef u_char  message_type;
-#define	MSG_STORE	1
-#define MSG_READ	2
-#define MSG_ALERT	3
-#define MSG_COMMAND	4
-#define MSG_PVM_BILLING	5
+#define MSG_STORE	0
+#define MSG_READ	1
+#define MSG_ALERT	2
+#define MSG_COMMAND	3
+#define MSG_ACLSERVER	5
 
 
 class Message {
 	private:
-		pthread_mutex_t *lock;
+		pthread_mutex_t lock;
 	public:
 		u_char active;
 		Message *next;
 		message_type type;
 		Message(message_type t);
-		~Message();
+		virtual ~Message();
 		void Push();
 		void Pop();
-		u_char Active();
 };
 
 class Message_Read: public Message {
@@ -41,7 +40,6 @@ class Message_Read: public Message {
 		oid netunit;
 		oid ap;
 		policy_data *pdata;
-		char prefix; // M, W, D, H, F
 		Message_Read();
 		~Message_Read();
 };
@@ -51,7 +49,7 @@ class Message_Store: public Message {
 		oid netunit;
 		oid ap;
 		pstat *data;
-		char prefix; // M, W, D, H, F
+		char prefix; // M, W, D, H, F, O
 		time_t ts;
 		Message_Store();
 		~Message_Store();
@@ -64,13 +62,12 @@ class Message_Alert: public Message {
 		~Message_Alert();
 };
 
-class Message_PVM_Billing: public Message {
+class Message_AclServer: public Message {
 	public:
-		Account *account;
-		char *text;
-		time_t ts;
-		Message_PVM_Billing();
-		~Message_PVM_Billing();
+		struct in_addr ip;
+		u_char flag;
+		Message_AclServer();
+		~Message_AclServer();
 };
 //////////////////////////////////////////////////////////////////////////
 typedef struct MsgHolder {
@@ -81,7 +78,7 @@ typedef struct MsgHolder {
 // FIFO class
 class FIFO {
 	private:
-		pthread_mutex_t *lock;
+		pthread_mutex_t lock;
 	public:
 		MsgHolder *root;
 		MsgHolder *last;
@@ -101,7 +98,7 @@ class FIFO {
 //////////////////////////////////////////////////////////////////////////
 class MessageManager {
 	private:
-		pthread_mutex_t *lock;
+		pthread_mutex_t lock;
 	public:
 		Message *ready[NUM_MSG_TYPES];
 		unsigned ready_items[NUM_MSG_TYPES];
@@ -113,26 +110,9 @@ class MessageManager {
 		~MessageManager();
 		Message* New(message_type t);
 		void Delete(Message *msg);
-		void Usage(Connection *conn);
+		void Usage(struct cli_def *cli);
 };
 //////////////////////////////////////////////////////////////////////////
-class DS_FIFO {
-	public:
-		IPStatMsg *root;
-		IPStatMsg *last;
-		IPStatMsg *ready;
-		unsigned num_items;
-		unsigned max_items;
-		unsigned long total_items;
-		unsigned ready_items;
-		pthread_mutex_t *lock;
-                DS_FIFO(unsigned max);
-                ~DS_FIFO();
-                IPStatMsg* Push(IPStatMsg *msg);
-                IPStatMsg* Pop(IPStatMsg *msg);
-};
-//////////////////////////////////////////////////////////////////////////
-//////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////
  
