@@ -7,7 +7,7 @@
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: s_quota.c,v 1.129 2009-08-01 09:23:55 anton Exp $ */
+/* $Id: s_quota.c,v 1.130 2009-09-03 13:30:47 jura Exp $ */
 
 #include "netams.h"
 
@@ -270,8 +270,12 @@ void Service_Quota::RestoreQuota() {
 	netams_rwlock_rdlock(&Units->rwlock);
 	for (u=(NetUnit*)Units->root; u!=NULL; u=(NetUnit*)u->next) {
 		for(q=u->quotadata; q!=NULL; q=q->next) {
-			if(q->flags&QUOTA_BLOCKED)
-				QuotaAction(u, q, ADD, now);
+			if(q->flags&QUOTA_ACTIVE) {
+				if(q->flags&QUOTA_BLOCKED)
+					QuotaAction(u, q, ADD, now);
+				else
+					QuotaAction(u, q, REMOVE, now);
+			}
 			q->flags&=~QUOTA_UPDATE;
 			aDebug(DEBUG_QUOTA, "Unit: %06X soft %u act %u blo %u softblo %u\n",
 				u->id, q->soft_treshold,
