@@ -7,7 +7,7 @@
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: security.c,v 1.59 2009-08-01 09:23:54 anton Exp $ */
+/* $Id: security.c,v 1.61 2009-12-19 17:25:50 anton Exp $ */
 
 #include "netams.h"
 #include "ds_raw.h"
@@ -22,23 +22,23 @@ extern "C" {
 /////////////////////////////////////////////////////////////////////////////////////////
 // Permissions Management ///////////////////////////////////////////////////////////////////
 
-const char *permissions_list[256]; 
+const char *permissions_list[256];
 
 void aPermissionsInit(){
 
 	for (unsigned i=0; i<=255; i++) permissions_list[i]=NULL;
-	permissions_list[UPERM_NONE]="none";              
-	permissions_list[UPERM_LOGIN_WEB]="weblogin";          
+	permissions_list[UPERM_NONE]="none";
+	permissions_list[UPERM_LOGIN_WEB]="weblogin";
 	permissions_list[UPERM_SHOWSTAT_HIS]="showstat_his";
-	permissions_list[UPERM_SHOWOBJ_HIS]="showobj_his";      
-	permissions_list[UPERM_MODIFYOBJ_HIS]="modifyobj_his";    
+	permissions_list[UPERM_SHOWOBJ_HIS]="showobj_his";
+	permissions_list[UPERM_MODIFYOBJ_HIS]="modifyobj_his";
 	permissions_list[UPERM_SHOWSTAT_ALL]="showstat";
-	permissions_list[UPERM_SHOWOBJ_ALL]="showobj";          
-	permissions_list[UPERM_SHOWCONFIG]="showconfig";       
-	permissions_list[UPERM_RADIUS]="radius";       
+	permissions_list[UPERM_SHOWOBJ_ALL]="showobj";
+	permissions_list[UPERM_SHOWCONFIG]="showconfig";
+	permissions_list[UPERM_RADIUS]="radius";
 	permissions_list[UPERM_MODIFYOBJ_ALL]="modifyobj";
-	permissions_list[UPERM_MODIFYCONFIG]="modifyconfig";     
-	permissions_list[UPERM_LOGIN_INT]="intlogin";        
+	permissions_list[UPERM_MODIFYCONFIG]="modifyconfig";
+	permissions_list[UPERM_LOGIN_INT]="intlogin";
 	permissions_list[UPERM_SYSACTION]="sysaction";
 	permissions_list[UPERM_ALL]="all";
 
@@ -54,14 +54,14 @@ char* CryptWord(char *s) {
 	char *crypted;
 	crypted = crypt(s, "$1$");
 	return set_string(crypted);
-}	
+}
 //////////////////////////////////////////////////////////////////////////
 char *enable_password = NULL;
 
 int cEnable		(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 	u_char no_flag;
 	u_char i=0;
-		
+
 	no_flag = CLI_CHECK_NO(argv[i]);
 	if(no_flag) {
 		if(enable_password) aFree(enable_password);
@@ -69,34 +69,34 @@ int cEnable		(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 		return CLI_OK;
 	}
  	i++;
- 	
- 	if (STRARG(argv[i], "crypted")) { 
+
+ 	if (STRARG(argv[i], "crypted")) {
 		if(enable_password) aFree(enable_password);
 		enable_password=set_string(argv[i+1]);
  	} else if (STRARG(argv[i], "password")) {
 		if(enable_password) aFree(enable_password);
 		enable_password=CryptWord(argv[i+1]);
-	} else 
+	} else
 		return CLI_ERROR;
-		
+
 	cli_error(cli, "enable crypted set: %s", enable_password);
-				
-	return CLI_OK;		
+
+	return CLI_OK;
 }
 
 int aEnable(char *s) {
 	if(enable_password == NULL) return 1;
 	int res=0;
-	
+
 	char *crypted=CryptWord(s);
 	if (STREQ(enable_password, crypted))
 		res = 1;
 	aFree(crypted);
-	
+
 	return res;
 }
 
-//////////////////////////////////////////////////////////////////////////		
+//////////////////////////////////////////////////////////////////////////
 u_char aCheckPerm(Connection *conn, u_char perm){
 	if (conn->permissions < perm) {
 		aLog(D_WARN, "connection %u: not enough permissions\n", conn->id);
@@ -107,13 +107,13 @@ u_char aCheckPerm(Connection *conn, u_char perm){
 //////////////////////////////////////////////////////////////////////////
 int aAuth(char *login, char *password){
 	User *u;
-		
+
 	if (!login || !password) return -1;
-	
+
 	u=Users->getUser(login);
-	
+
 	if (!u) return 0;
-	
+
 	int res=0;
 
 #ifdef HAVE_PAM
@@ -138,10 +138,10 @@ int aAuth(char *login, char *password){
 
 static int password_conversation(int num_msg, const struct pam_message **msg, struct pam_response **resp, void *appdata_ptr)
 {
-    if (num_msg != 1 || msg[0]->msg_style != PAM_PROMPT_ECHO_OFF) 
+    if (num_msg != 1 || msg[0]->msg_style != PAM_PROMPT_ECHO_OFF)
 	return PAM_CONV_ERR;
 
-    if (!appdata_ptr) 
+    if (!appdata_ptr)
 	return PAM_CONV_ERR;
 
     *resp = (pam_response*)calloc(num_msg, sizeof(struct pam_response));
@@ -159,7 +159,7 @@ int aAuthPam(char *username, char *password){
 
 	if ( !username || !password )
 	    return 0;
-	    
+
 	static struct pam_conv conv = {
 		&password_conversation,
 		NULL
@@ -167,7 +167,7 @@ int aAuthPam(char *username, char *password){
 	conv.appdata_ptr = (char *) password;
 	pam_handle_t *pamh=NULL;
 	int retval;
-    
+
 	retval = pam_start("netams", username, &conv, &pamh);
 	if (retval == PAM_SUCCESS)
 		retval = pam_authenticate(pamh, 0);    /*   ? */
@@ -182,7 +182,7 @@ int aAuthPam(char *username, char *password){
 		aLog(D_INFO, "aAuthPam: Account for %s fail\n",username);
 END:
 	pam_end(pamh,retval);
-	
+
     return permissions;
 }
 #endif
@@ -198,12 +198,12 @@ u_char cGetUnitAndPass(char *login,int b_login,NetUnit *&u,char *&password) {
 		Account *ac;
 		ac = bAccounts->Get(login);
 		if (ac) {
-			u = ac -> bUroot -> u;
+			if ( ac -> bUroot != NULL) u = ac -> bUroot -> u;
 			password = ac -> password;
 			return 1;
 		}
-	} else 
-#endif	
+	} else
+#endif
 	{
 		u = Units -> getUnit(login);
 		if (u){
@@ -211,7 +211,7 @@ u_char cGetUnitAndPass(char *login,int b_login,NetUnit *&u,char *&password) {
 			return 1;
 		}
 	}
-	return 0;	
+	return 0;
 }
 //////////////////////////////////////////////////////////////////////////
 int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
@@ -223,12 +223,12 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 	struct in_addr framed_ip; framed_ip.s_addr=INADDR_ANY;
 	char *options=NULL;
 	char buffer[32];
-	
+
 	for(i++; i< argc; i+=2)  {
 		if (STRARG(argv[i],"account")) {
 				billing_login=1;
 				login=set_string(argv[i+1]);
- 			} else if (STRARG(argv[i], "login")) { 
+ 			} else if (STRARG(argv[i], "login")) {
 				billing_login=0;
 				login=set_string(argv[i+1]);
 			} else if (STRARG(argv[i], "password")) {
@@ -259,7 +259,7 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 						int num_options=0;
 						struct ether_addr *ether_callback=NULL;
 						if (callback_id) ether_callback=ether_aton(callback_id);
-						 
+
 						if (u->type==NETUNIT_USER) {
 							NetUnit_user *us = (NetUnit_user*)u;
 							if (us->ip.s_addr!=0) {
@@ -271,7 +271,7 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 							if (us->ip.s_addr!=0 && nas_ip.s_addr!=INADDR_ANY) {
 								if ((nas_ip.s_addr&0x0000ffff) != (us->ip.s_addr&0x0000ffff)) fail=1;
 							}
-								
+
 						}
 						else if (u->type==NETUNIT_HOST) {
 							NetUnit_host *uh = (NetUnit_host*)u;
@@ -295,7 +295,7 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 							print_to_string(&options, "User-Password: %s\n", u_pass);
 						}
 						// add Session-Timeout handler here
-						
+
 						if (fail) {
 							cli_error(cli, "0 callback check failed for %s", login);
 							LogEvent(RADIUS, u->id, 0, 0, "FAIL callback check for %s, nas-id %s callback-id %s", login, nas_id?nas_id:"<none>", callback_id?callback_id:"<none>");
@@ -305,7 +305,7 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 							LogEvent(RADIUS, u->id, 0, 0, "SUCCESS login %s nas-id %s callback-id %s", login, nas_id?nas_id:"<none>", callback_id?callback_id:"<none>");
 							aLog(D_INFO, "RADIUS SUCCESS login %s nas-id %s callback-id %s\n", login, nas_id?nas_id:"<none>", callback_id?callback_id:"<none>");
 						}
-					} else { 
+					} else {
 						cli_error(cli, "0 password incorrect for %s", login);
 						LogEvent(RADIUS, u->id, 0, 0, "FAIL password incorrect for %s, nas-id %s callback-id %s", login, nas_id?nas_id:"<none>", callback_id?callback_id:"<none>");
 						aLog(D_INFO, "RADIUS FAIL password incorrect for %s, nas-id %s callback-id %s\n", login, nas_id?nas_id:"<none>", callback_id?callback_id:"<none>");
@@ -326,7 +326,7 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 			char *u_pass=NULL;
 			NetUnit *u=NULL;
 			if (cGetUnitAndPass(login, billing_login, u, u_pass)) {
-			//NetUnit *u = Units->getUnit(login);		
+			//NetUnit *u = Units->getUnit(login);
 				if (u) {
 					if ( (u_pass==NULL && (password==NULL) ) ||
 						STREQ(u_pass, password)) {
@@ -350,9 +350,9 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 		else
 			cli_error(cli, "0 radius authorization source is unknown: %s", argv[2]);
 
-	} 
+	}
 	else if (STREQ(argv[1], "acct") && login) {
-		pstat data; 
+		pstat data;
 		Policy *p=NULL;
 		i=3;
 		bzero(&data, sizeof (pstat));
@@ -360,9 +360,9 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 		if (!u) cli_error(cli, "0 radius accounting login is unknown: %s", login);
 		else {
 			for(i++; i< argc; i+=2)  {
-		 		if (STRARG(argv[i], "in")) 
+		 		if (STRARG(argv[i], "in"))
 					data.in=bytesT2Q(argv[i+1]);
-		 		else if (STRARG(argv[i], "out"))  
+		 		else if (STRARG(argv[i], "out"))
 					data.out=bytesT2Q(argv[i+1]);
 		 		else if (STRARG(argv[i], "policy"))  {
 		 			i++;
@@ -371,7 +371,7 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 		 			}
 		 		else i--;
 			}
-	
+
 			aLog(D_INFO, "RADIUS ACCT (%s) for %s(%s)/%06X, nas-id %s, in %llu, out %llu\n", argv[2], login, framed_ip.s_addr!=INADDR_ANY?inet_ntoa(framed_ip):"no IP given", u->id, nas_id?nas_id:"<none>", data.in, data.out);
  			if (u->type==NETUNIT_USER) {
 				if (STREQ(argv[2], "start") && framed_ip.s_addr!=INADDR_ANY) {
@@ -401,9 +401,9 @@ int cRadius(struct cli_def *cli, const char *cmd, char **argv, int argc) {
 				memcpy(&raw.data, &data, sizeof(pstat));
 				cRawData_prepared(cli, &raw);
 			}
-		cli_error(cli, "1 radius accounting"); 
+		cli_error(cli, "1 radius accounting");
 		}
-	} 
+	}
 
 	aFree(login); aFree(password); aFree(nas_id); aFree(callback_id); aFree(options);
 	return CLI_OK;
@@ -417,7 +417,6 @@ int cPam(struct cli_def *cli, const char *cmd, char **param, int argc){
 	char *password=NULL;
 	char *p_password=NULL;
 	char *p_name=NULL;
-	char *p_resp=NULL;
 	u_char i=2;
 	u_char check_of_whom=0; // 1-unit; 2-user; 3-account; 4-login
 	u_char check_ok=0;
@@ -458,7 +457,7 @@ int cPam(struct cli_def *cli, const char *cmd, char **param, int argc){
 	}
 #endif
 
-	if ( !p_name || !check_of_whom || 
+	if ( !p_name || !check_of_whom ||
 		    !( STREQ(param[1], "auth") || STREQ(param[1], "acct")) ||
 		     ( STREQ(param[1], "auth") && !p_password ) ) {
 
@@ -523,7 +522,7 @@ int cPam(struct cli_def *cli, const char *cmd, char **param, int argc){
 			cli_error(cli, "-1 PAM: account %s beblocked", p_name);
 		    else {
 			    check_ok=1;
-			    if ( STREQ(param[1], "auth" ) ) 
+			    if ( STREQ(param[1], "auth" ) )
 				password = ac -> password;
 		    }
 
@@ -534,7 +533,7 @@ int cPam(struct cli_def *cli, const char *cmd, char **param, int argc){
 #endif // BILLING
 
 	aLog(D_INFO, "PAM_netams check %s %s %s\n",check_ok?"Ok":"Fail",param[2],p_name);
-	
+
 	if ( check_ok ) {
 
 		if (STREQ(param[1], "acct"))
@@ -550,7 +549,7 @@ int cPam(struct cli_def *cli, const char *cmd, char **param, int argc){
 				aLog(D_INFO, "PAM_netams auth Fail: %s name %s pass %s\n",param[2],p_name,p_password);
 			}
 		}
-	}	
+	}
 
 #endif // HAVE_PAM
 
