@@ -1,19 +1,42 @@
 /*************************************************************************
 ***	Authentication, authorization, accounting + firewalling package
-***	(c) 1998-2002 Anton Vinokurov <anton@netams.com>
-***	(c) 2002-2005 NeTAMS Development Team
-***	All rights reserved. See 'Copying' file included in distribution
+***	Copyright 1998-2002 Anton Vinokurov <anton@netams.com>
+***	Copyright 2002-2008 NeTAMS Development Team
+***	This code is GPL v3
 ***	For latest version and more info, visit this project web page
 ***	located at http://www.netams.com
 ***
 *************************************************************************/
-/* $Id: st_sql_oracle.c,v 1.3.2.2 2005/02/18 18:03:28 anton Exp $ */
+/* $Id: st_sql_oracle.c,v 1.34 2009-08-01 09:23:55 anton Exp $ */
 
 #ifdef USE_ORACLE
 
 #include "netams.h"
-#include "oci.h"
+#include <oci.h>
+#include "st_any.h"
 
+//////////////////////////////////////////////////////////////////////////
+void *ora_stOpenSql(struct sql_config *cfg,st_conn_type type);
+void *ora_stCheckSql(void *fd);
+unsigned ora_stSaveSql(void *fd, char *filename, st_conn_type type);
+void ora_stCloseSql(void *fd);
+u_char ora_stObtainDbData(void *fd, char *query,
+	void (*FillData)(void*, void* ,char* (*getRowData)(void*, void* , u_char)));
+//////////////////////////////////////////////////////////////////////////
+struct sql_data ora_sql_data = {
+	0,
+	&ora_stOpenSql,
+	&ora_stCheckSql,
+	&ora_stSaveSql,
+	&ora_stCloseSql,
+	&ora_stObtainDbData
+};
+//////////////////////////////////////////////////////////////////////////
+char* ora_getRowData(void *res, void *row, u_char num) {
+	/* aLog(DEBUG_STORAGE, "getRowData() = %s\n", (*(char ***)row)[num]); */
+	return (*(char ***)row)[num];
+}
+//////////////////////////////////////////////////////////////////////////
 /*
  * OCI is bit different from MySQl/PG in programming model.
  * First, we must allocate OCI environment, in which all
@@ -30,12 +53,12 @@ static OCIEnv *g_env;
 static pthread_mutex_t g_env_mtx = PTHREAD_MUTEX_INITIALIZER;
 /* here goes Oracle version */
 char    oraver[512];
-   
+
 typedef struct oracle_conn {
     OCISvcCtx *svcctx;
     OCIError *errh;
 	OCIStmt  *stmth;
-    OCIServer *srvh; 
+    OCIServer *srvh;
     char **row;
 } ORAconn;
 
@@ -57,20 +80,20 @@ static void ora_checkerr(dvoid *errhp, ub4 htype, sword status,
 #define OCI_CHECK(errhp, htype, status, ctlp, OCIfunc) \
 if (OCI_SUCCESS != ((status) = (OCIfunc))) \
 { \
-  checkerr((dvoid *)(errhp), (ub4)(htype), (sword)(status), (text *)0, \
-           (sb4)0, (text *)__FILE__, (sb4)__LINE__); \
+  checkerr((dvoid *)(errhp), (ub4)(htype), (sword)(status), NULL, \
+           NULL, (text *)__FILE__, (sb4)__LINE__); \
   if ((status) != OCI_SUCCESS_WITH_INFO) \
     cleanup((struct loadctl *)ctlp, (sb4)1); \
 } else
 
 #define OCI_CHK(status, errhp)                                \
-  ora_checkerr((dvoid *)errhp, (ub4)(OCI_HTYPE_ERROR), (sword)(status), (text *)0, \
-           (sb4)0, (text *)__FILE__, (sb4)__LINE__);
+  ora_checkerr((dvoid *)errhp, (ub4)(OCI_HTYPE_ERROR), (sword)(status), NULL, \
+		  0, (text *)__FILE__, (sb4)__LINE__);
 
 #define FATAL(note, state) \
 do \
 { \
-  checkerr((dvoid *)0, (ub4)OCI_HTYPE_ERROR, (sword)OCI_SUCCESS,           \
+  checkerr(NULL, (ub4)OCI_HTYPE_ERROR, (sword)OCI_SUCCESS,           \
            (text *)(note), (sb4)(state), (text *)__FILE__, (sb4)__LINE__); \
   cleanup((ctlp), (sb4)2); \
 } while (0)
@@ -78,9 +101,8 @@ do \
 
 //////////////////////////////////////////////////////////////////////////
 void *
-ora_stOpenSql(Service *s, st_conn_type type)
+ora_stOpenSql(struct sql_config *cfg, st_conn_type type)
 {
-	ServiceStorage_cfg *cfg = (ServiceStorage_cfg *)s->cfg;
 //	FILE *debug;
     ORAconn *conn;
 	char *st_conn = NULL;
@@ -96,7 +118,7 @@ ora_stOpenSql(Service *s, st_conn_type type)
     }
 
     /* check if this is first call to OCI */
-    pthread_mutex_lock(&g_env_mtx);
+    netams_mutex_lock(&g_env_mtx);
     if (g_env == NULL) {
         /* create OCI environment */
         if (OCIEnvCreate(&g_env, OCI_THREADED,
@@ -105,22 +127,11 @@ ora_stOpenSql(Service *s, st_conn_type type)
             return (NULL);
         }
     }
-    pthread_mutex_unlock(&g_env_mtx);
+    netams_mutex_unlock(&g_env_mtx);
 
-    if (!cfg->username)
-        ora_user="netams";
-    else
-        ora_user=cfg->username;
-
-    if (!cfg->password)
-        ora_pass="netams";
-    else
-        ora_pass=cfg->password;
-
-    if (!cfg->dbname)
-        ora_dbname="netams";
-    else
-        ora_dbname=cfg->dbname;
+    ora_user=cfg->username;
+    ora_pass=cfg->password;
+    ora_dbname=cfg->dbname;
 
     conn = (ORAconn *)aMalloc(sizeof(ORAconn));
 
@@ -134,7 +145,7 @@ ora_stOpenSql(Service *s, st_conn_type type)
 //    rc = OCIServerAttach(conn->srvh, conn->errh,
 //        (CONST text*)ora_dbname, strlen(ora_dbname), OCI_DEFAULT);
     rc = OCIServerAttach(conn->srvh, conn->errh,
-        (CONST text*)0, 0, OCI_DEFAULT);
+        NULL, 0, OCI_DEFAULT);
     OCI_CHK(rc, conn->errh);
 
     /* try connect to DB */
@@ -148,7 +159,7 @@ ora_stOpenSql(Service *s, st_conn_type type)
     rc = OCIServerVersion(conn->srvh, conn->errh, (text*)oraver,
         sizeof(oraver), OCI_HTYPE_SERVER);
     OCI_CHK(rc, conn->errh);
-    
+
     /* create statement handle */
     rc = OCIHandleAlloc(g_env, (dvoid **)&conn->stmth,
         OCI_HTYPE_STMT, 0, NULL);
@@ -164,7 +175,12 @@ ora_stOpenSql(Service *s, st_conn_type type)
 	//res=PQexec(conn,"BEGIN");PQclear(res);
 	return (void*)conn;
 }
-
+//////////////////////////////////////////////////////////////////////////
+void
+*ora_stCheckSql(void *fd) {
+	//put something here
+	return fd;
+}
 //////////////////////////////////////////////////////////////////////////
 void
 ora_stCloseSql(void *fd)
@@ -178,7 +194,7 @@ ora_stCloseSql(void *fd)
     OCIHandleFree(conn->srvh, OCI_HTYPE_SERVER);
     	if (conn->row != NULL)
         	ora_free_row(conn->row);
-	
+
 	aFree(conn);
 
 }
@@ -208,7 +224,7 @@ ora_stSaveSql(void *fd, char *filename, st_conn_type type)
             aFree(values);
             lines ++;
             if ((rc = ora_execute(conn, q)) != OCI_SUCCESS) {
-                aLog(D_WARN, "ora_stSaveSql() failed for %s\n", q);
+                aLog(D_ERR, "ora_stSaveSql() failed for %s\n", q);
                 OCI_CHK(rc, conn->errh);
                 continue;
             }
@@ -220,7 +236,7 @@ ora_stSaveSql(void *fd, char *filename, st_conn_type type)
 
 		// commit changes
         if ((rc = ora_execute(conn, "COMMIT")) != OCI_SUCCESS) {
-            aLog(D_WARN, "ora_stSaveSql() failed for COMMIT\n");
+            aLog(D_ERR, "ora_stSaveSql() failed for COMMIT\n");
             OCI_CHK(rc, conn->errh);
         }
 
@@ -234,290 +250,22 @@ ora_stSaveSql(void *fd, char *filename, st_conn_type type)
 
 //////////////////////////////////////////////////////////////////////////
 u_char
-ora_stLoadSql(void *fd, Message *message)
-{
-	Message_Read *msg=(Message_Read *)message;
-	char query[255], **row;
-	ORAconn *conn = (ORAconn*)fd;
-	pstat *cps=NULL;
-    sword rc;
+ora_stObtainDbData(void *fd, char *query,
+	void (*FillData)(void*, void* ,char* (*)(void*, void* , u_char))) {
 
-	//actually there is no "F"low read requests
-	switch (msg->prefix){
-    case 'M': cps=&(msg->pdata->m); break;
-    case 'W': cps=&(msg->pdata->w); break;
-    case 'D': cps=&(msg->pdata->d); break;
-    case 'H': cps=&(msg->pdata->h); break;
-    }
-
-	snprintf(query, 254, "SELECT bytes_in, bytes_out FROM summary "
-        "WHERE prefix='%c' AND unit_oid=%u AND policy_oid=%u AND t_from=%lu",
-		msg->prefix, msg->netunit, msg->ap, (unsigned long)cps->from);
-	rc = ora_select(conn, query);
-    if (rc == OCI_SUCCESS) {
-        if ((row = ora_fetch(conn)) != NULL) {
-            unsigned long long    in, out;
-            in = strtoull(conn->row[0], NULL, 0);
-            out = strtoull(conn->row[1], NULL, 0);
-            cps->in += in;
-            cps->out += out;
-        }
-    }
-
-	aDebug(DEBUG_STORAGE, "SQL<-HDD/%c query %u bytes, fr=%lu in=%llu out=%llu\n",
-        msg->prefix, strlen(query), cps->from, cps->in, cps->out);
-
-	return 1;
-}
-
-//////////////////////////////////////////////////////////////////////////
-u_char
-ora_stLgObtainDbData(void *fd)
-{
-	char *buffer;
 	ORAconn *conn = (ORAconn *)fd;
 
-	buffer="SELECT * from login";
-	aDebug(DEBUG_LOGIN, "sLgObtainDbData query: '%s'\n", buffer);
-	if (ora_select(conn, buffer) != OCI_SUCCESS) { 
-		return 0; 
-	}
-
-    {	
-        char **row;
-		sLoginData *logindata;
-		unsigned tmp;
-		NetUnit *u=NULL;
-		oid id;
-		
-		while((row=ora_fetch(conn)) != NULL){
-			sscanf(row[0], "%u", &id);
-			if(id && !(u=Units.getUnitById(id))) continue;
-			if(u->logindata) {
-				logindata=u->logindata;
-				aFree(logindata->password);
-			} else 
-				logindata = (sLoginData*)aMalloc(sizeof(sLoginData));		
-
-			logindata->password=set_string(row[1]);
-            sscanf(row[2], "%lu", (unsigned long*)&logindata->inact);
-            sscanf(row[3], "%lu", (unsigned long*)&logindata->abs);
-            sscanf(row[5], "%lu", (unsigned long*)&logindata->opened);
-            sscanf(row[6], "%u", &logindata->ip_from.s_addr);
-			memcpy(&logindata->mac_from, ether_aton(row[7]),
-                sizeof (struct ether_addr));
-            sscanf(row[8], "%u", &tmp); logindata->flags=tmp;
-			
-			u->logindata=logindata;
-		}
-    }
-	return 1;
-}
-//////////////////////////////////////////////////////////////////////////
-u_char
-ora_stQuObtainDbData(void *fd)
-{
-    char *buffer, **row;
-	ORAconn *conn = (ORAconn*)fd;
-    sQuotaData *q;
-    unsigned tmp;
-    NetUnit *u=NULL;
-    oid id;
-
-	buffer="SELECT * from quota";
-    aDebug(DEBUG_QUOTA, "sQuObtainDbData query: '%s'\n", buffer);
-	if (ora_select(conn, buffer) != OCI_SUCCESS) {
+	if (ora_select(conn, query) != OCI_SUCCESS) {
 		return 0;
 	}
 
-    while((row=ora_fetch(conn)) != NULL) {
-        sscanf(row[0], "%u", &id);
-        if(id && !(u=Units.getUnitById(id))) continue;
-        if(u->quotadata) {
-            q=u->quotadata;
-        } else
-            q = (sQuotaData*)aMalloc(sizeof(sQuotaData));
-                       	
-        sscanf(row[1], "%u", &id); q->policy=PolicyL.getPolicyById(id);
-        sscanf(row[3], "%u", &tmp); q->soft_treshold=tmp;
-        sscanf(row[4], "%u", &tmp); q->flags=tmp;
-        sscanf(row[5], "%lu", (unsigned long*)&q->blocked_time);
+       	char **row;
+	while((row=ora_fetch(conn)) != NULL)
+		FillData( NULL, &row, &ora_getRowData);
 
-        sscanf(row[6], "%u", &q->nso);
-        sscanf(row[7], "%u", &q->nho);
-        sscanf(row[8], "%u", &q->nro);
-
-        sscanf(row[9], "%llu", &q->h.in);
-        sscanf(row[10], "%llu", &q->h.out);
-        sscanf(row[11], "%llu", &q->h.sum);
-
-        sscanf(row[12], "%llu", &q->d.in);
-        sscanf(row[13], "%llu", &q->d.out);
-        sscanf(row[14], "%llu", &q->d.sum);
-
-        sscanf(row[15], "%llu", &q->w.in);
-        sscanf(row[16], "%llu", &q->w.out);
-        sscanf(row[17], "%llu", &q->w.sum);
-
-        sscanf(row[18], "%llu", &q->m.in);
-        sscanf(row[19], "%llu", &q->m.out);
-        sscanf(row[20], "%llu", &q->m.sum);
-		
-		sscanf(row[21], "%u", &id); if (id) q->fw_block_policy=PolicyL.getPolicyById(id); else q->fw_block_policy=NULL;
-		sscanf(row[22], "%u", &tmp); q->fw_block_policy_flags=tmp;
-			
-        u->quotadata=q;
-    } 
-    return 1;
-}
-
-//////////////////////////////////////////////////////////////////////////
-#ifdef HAVE_BILLING
-
-u_char ora_stBiObtainDbData(void *fd){
-	char *buffer, **row;
-	ORAconn *conn = (ORAconn*)fd;
-	
-	aDebug(DEBUG_BILLING, "SQL->HDD/billing data request\n");
-	buffer="SELECT * FROM billing ORDER BY CREATED";
-	if (ora_select(conn, buffer) != OCI_SUCCESS) {
-		aLog(D_WARN, "oracle select billing failed: \n"); 
-		return 0; 
-	}
-
-	Account *ac;
-	NetUnit *u;
-	unsigned t;
-    while((row=ora_fetch(conn)) != NULL) {
-
-			sscanf(row[0], "%u", &t);
-			newOid(t);
-			
-			sscanf(row[10], "%u", &t); //created ?
-			if(!t) continue; //this account deleted
-
-			sscanf(row[0], "%u", &t);
-			ac=new Account();
-			ac->id=newOid(t);
-
-//			for(int j=0; j<16; j++) printf("row[%d]=%s\n", j, row[j]); 
-			if (row[1][0]) ac->name=set_string(row[1]);
-			if (row[2][0]) ac->description=set_string(row[2]);
-			
-			sscanf(row[3], "%lf", &ac->balance);
-			
-			sscanf(row[5], "%u", &t); ac->plan=bPlans->Check(t);
-			sscanf(row[6], "%lu", (unsigned long*)&ac->plan_ch);
-			sscanf(row[7], "%u", &t); ac->nextplan=bPlans->Check(t);
-			sscanf(row[8], "%lu", (unsigned long*)&ac->nextplan_ch);
-			sscanf(row[9], "%lu", (unsigned long*)&ac->blocked);
-			sscanf(row[10], "%lu", (unsigned long*)&ac->created);
-			sscanf(row[11], "%lu", (unsigned long*)&ac->changed);
-			sscanf(row[12], "%lu", (unsigned long*)&ac->last_fee_ch);
-							  
-			if (row[13][0]) ac->email=set_string(row[13]);
-			if (row[14][0]) ac->password=set_string(row[14]);
-			sscanf(row[15], "%u", &t); ac->status=t;
-			
-			u_short i=0,k=strlen(row[4]);
-			while (i*7<k) {
-				sscanf(row[4]+i*7, "%06X", &t);
-				if ((u=Units.getUnitById(t))) ac->AddUnit(u, ADD); else break;
-				i++;
-			}
-
-			bAccounts->Add(ac);
-	}
 	return 1;
 }
-
-extern struct FeeCounters FC;
-
-void ora_stBiLoadBdata(void *fd, Account *ac, char prefix) {
-	char query[255];
- 	
-	char **row;
-	ORAconn *conn = (ORAconn*)fd;
-	
-	bstat *bs=NULL;
-	time_t t_from=0;
-	
-	switch (prefix){
-		case 'M': t_from=FC.mt; break;
-		case 'W': t_from=FC.wt; break;
-		case 'D': t_from=FC.dt; break;
-		case 'H': t_from=FC.ht; break;
-        }
-
-	snprintf(query, 254, "SELECT subplan_oid, bytes_in, bytes_out, pay_in, pay_out FROM bdata WHERE account_oid=%u AND t_from=%lu AND prefix='%c'", ac->id, (unsigned long)t_from, prefix);
-	if (ora_select(conn, query) != OCI_SUCCESS) {
-		aLog(D_WARN, "oracle select bstat failed: \n"); 
-		return; 
-	}
-	
-	bSPlist *bsp;
-	unsigned id;
-	u_char i;
-
-    while((row=ora_fetch(conn)) != NULL) {
-			sscanf(row[0], "%u", &id);
-			
-			for(i=0,bsp=ac->plan->root;bsp!=NULL;bsp=bsp->next) {
-				if(bsp->sp->id==id) {
-					switch (prefix){
-						case 'M': bs=&ac->data[i].m; break;
-						case 'W': bs=&ac->data[i].w; break;
-						case 'D': bs=&ac->data[i].d; break;
-						case 'H': bs=&ac->data[i].h; break;
-					}
-					bs->from=t_from;
-
-					sscanf(row[1], "%lld", &bs->in);
-					sscanf(row[2], "%lld", &bs->out);
-					sscanf(row[3], "%lf", &bs->pay_in);
-					sscanf(row[4], "%lf", &bs->pay_out);
-					break;
-				}
-				i++;
-			}
-		}
-	aDebug(DEBUG_BILLING, "Account %s(%06X), '%c' bstat loaded\n", ac->name, ac->id, prefix);
-}
-#endif // billing
-
 //////////////////////////////////////////////////////////////////////////
-// SQL config
-/* XXX broken in -stable
-void
-ora_stConfigGet(void *fd, config_entry *e, schema_entry *h)
-{
-	char query[255], **row;
-	ORAconn *conn = (ORAconn*)fd;     
-	
-	snprintf(query, 254, "SELECT schema.name,config.id,i_val,s_val,optional "
-        "FROM config,schema "
-        "WHERE schema.skey=%d AND config.skey=schema.skey "
-        "ORDER BY config.skey,config.sequence limit 1",
-        e->skey);
-	if(ora_select(conn, query) != OCI_SUCCESS) {
-		return;
-	}
-    while ((row = ora_fetch(conn)) != NULL) {
-        h->name=(char*)aMalloc(strlen(row[0]));
-        strcpy(h->name, row[0]);
-        sscanf(row[1], "%d", &e->id);
-        sscanf(row[2], "%d", &e->i_val);
-        e->s_val=(char*)aMalloc(strlen(row[3]));
-        strcpy(e->s_val, row[3]);
-    }
-    aDebug(DEBUG_PARSE, "key %d read from config database\n", e->skey);
-}
-
-void
-ora_stConfigPut(Service *s, config_entry *e)
-{
-}
-*/
 /*
  * Oracle-specific functions.
  */
@@ -569,7 +317,7 @@ ora_conv_to_values(char *line)
     strcat(values, tmp);
     free(tmp);
 
-    return (values);    
+    return (values);
 }
 
 /*
@@ -586,6 +334,7 @@ ora_execute(ORAconn *conn, char *query)
     OCI_CHK(rc, conn->errh);
     rc = OCIStmtExecute(conn->svcctx, conn->stmth, conn->errh,
         1, 0, NULL, NULL, 0);
+    /* aDebug(DEBUG_STORAGE, "OCIStmtExecute() rc =%d\n", rc); */
     OCI_CHK(rc, conn->errh);
     return (rc);
 }
@@ -606,7 +355,7 @@ ora_select(ORAconn *conn, char *query)
 	rc = OCIStmtPrepare(conn->stmth, conn->errh,
         (CONST text*)query, strlen(query), OCI_NTV_SYNTAX, OCI_DEFAULT);
     OCI_CHK(rc, conn->errh);
-    
+
     /*
      * Don't prefetch rows since we will defined
      * output vars later.
@@ -636,9 +385,13 @@ ora_fetch(ORAconn *conn)
         return (NULL);
 
     /* data goes to conn->row */
-	rc = OCIStmtFetch(conn->stmth, conn->errh, 1, OCI_FETCH_NEXT, OCI_DEFAULT);
-    if (rc == OCI_SUCCESS)
+    rc = OCIStmtFetch2(conn->stmth, conn->errh, 1, OCI_FETCH_NEXT, 1, OCI_DEFAULT);
+    /* aDebug(DEBUG_STORAGE, "fetch got rc = %d\n", rc); */
+    /* OCI_CHK(rc, conn->errh); */
+    if (rc == OCI_SUCCESS){
+    /* aDebug(DEBUG_STORAGE, "fetch = %s\n", *(conn->row)); */
         return (conn->row);
+		}
     else
         return (NULL);
 }
@@ -661,14 +414,15 @@ ora_alloc_row(ORAconn *conn)
      */
     rc = OCIAttrGet(conn->stmth, OCI_HTYPE_STMT, (dvoid *)&col,
         0, OCI_ATTR_PARAM_COUNT, conn->errh);
+    /* aDebug(DEBUG_STORAGE, "got %d columns\n", col); */
     OCI_CHK(rc, conn->errh);
     row = (char **)aMalloc(sizeof(char *) * (col + 1));
     for (i = 0; i < col; i++) {
         rc = OCIParamGet(conn->stmth, OCI_HTYPE_STMT, conn->errh,
             (dvoid **)&param, i + 1);
         OCI_CHK(rc, conn->errh);
-        rc = OCIAttrGet((dvoid*)param, OCI_DTYPE_PARAM, 
-            (dvoid*)&dtype, (ub4*)0, OCI_ATTR_DATA_TYPE,
+        rc = OCIAttrGet((dvoid*)param, OCI_DTYPE_PARAM,
+            (dvoid*)&dtype, NULL, OCI_ATTR_DATA_TYPE,
             conn->errh);
         OCI_CHK(rc, conn->errh);
         dsize = DEF_DATASTR_LEN;
@@ -679,10 +433,11 @@ ora_alloc_row(ORAconn *conn)
          * for char/string type columns).
          */
         switch(dtype) {
+        case SQLT_AFC:
         case SQLT_CHR:
         case SQLT_STR:
             rc = OCIAttrGet((dvoid*)param, (ub4) OCI_DTYPE_PARAM,
-                (dvoid*) &dsize, (ub4 *)0, (ub4) OCI_ATTR_DATA_SIZE,
+                (dvoid*) &dsize, NULL, (ub4) OCI_ATTR_DATA_SIZE,
                 conn->errh);
                 OCI_CHK(rc, conn->errh);
                 /* FALLTHROUGH */
@@ -693,11 +448,14 @@ ora_alloc_row(ORAconn *conn)
         case SQLT_PDN:
         case SQLT_BIN:
         case SQLT_NUM:
-            row[i] = (char*)aMalloc(dsize + 1);
+        	/* aDebug(DEBUG_STORAGE, "allocate %d bytes for row %d\n", dsize+1, i); */
+        	row[i] = (char*)aMalloc(dsize + 1);
             break;
         default:
-            dsize = 0;
-            row[i]=NULL;
+        	aLog(D_ERR, "got unknown dtype=%d for row=%d\n", dtype, i);
+        	dsize = 0;
+        	// TODO: rewrite logic because lead to create core dump later in ora_fetch()
+        	row[i]=NULL;
             break;
         }
         rc = OCIDefineByPos(conn->stmth, &define, conn->errh, i + 1,
@@ -749,12 +507,12 @@ ora_errprint(dvoid *errhp, ub4 htype, sb4 *errcodep)
   {
     sb4  errcode;
 
-    if (errcodep == (sb4 *)0)
+    if (errcodep == NULL)
       errcodep = &errcode;
 
     (void) OCIErrorGet((dvoid *)errhp, (ub4) 1, (text *) NULL, errcodep,
                        errbuf, (ub4) sizeof(errbuf), htype);
-    aLog(D_WARN, "OCI Error - %.*s\n", 512, errbuf);
+    aLog(D_ERR, "OCI Error - %.*s\n", 512, errbuf);
   }
 }
 
@@ -767,32 +525,32 @@ ora_checkerr(dvoid *errhp, ub4 htype, sword status,
   if (status == OCI_SUCCESS)
       return;
 
-  (void) aLog(D_WARN, "OCI Error %ld occurred at File %s:%ld\n",
+  (void) aLog(D_ERR, "OCI Error %ld occurred at File %s:%ld\n",
       (long)status, (char *)file, (long)line);
 
   switch (status)
   {
   case OCI_SUCCESS_WITH_INFO:
-      (void) aLog(D_WARN, "Error - OCI_SUCCESS_WITH_INFO\n");
+      (void) aLog(D_ERR, "Error - OCI_SUCCESS_WITH_INFO\n");
     ora_errprint(errhp, htype, &errcode);
     break;
   case OCI_NEED_DATA:
-      (void) aLog(D_WARN, "Error - OCI_NEED_DATA\n");
+      (void) aLog(D_ERR, "Error - OCI_NEED_DATA\n");
     break;
   case OCI_NO_DATA:
-      (void) aLog(D_WARN, "Error - OCI_NODATA\n");
+      (void) aLog(D_ERR, "Error - OCI_NODATA\n");
     break;
   case OCI_ERROR:
     ora_errprint(errhp, htype, &errcode);
     break;
   case OCI_INVALID_HANDLE:
-      (void) aLog(D_WARN, "Error - OCI_INVALID_HANDLE\n");
+      (void) aLog(D_ERR, "Error - OCI_INVALID_HANDLE\n");
     break;
   case OCI_STILL_EXECUTING:
-    (void) aLog(D_WARN, "Error - OCI_STILL_EXECUTE\n");
+    (void) aLog(D_ERR, "Error - OCI_STILL_EXECUTE\n");
     break;
   case OCI_CONTINUE:
-    (void) aLog(D_WARN, "Error - OCI_CONTINUE\n");
+    (void) aLog(D_ERR, "Error - OCI_CONTINUE\n");
     break;
   default:
     break;
